using System;
using System.Collections;
using System.Collections.Specialized;
using System.Data;
using System.Data.OleDb;
using System.Net;
using Syslog.Config.SectionHandler;

namespace Syslog.Collector.Writer
{
    #region SimpleDbWriter comment
	/// <summary>
	/// Syslog Relay Controller 󂯎 <see cref="SyslogInfo" /> f[^x[Xɕۑ܂B
	/// </summary>
	/// <example>
	/// <para>App.config ݒB</para>
	/// <code>
	/// &lt;db
	///  connectionString="
	///         Provider=SQLOLEDB;
	///         Data Source=.;
	///         Initial Catalog=tempdb;
	///         Integrated Security=SSPI"
	///  table="##SyslogDetails"
	///  collectorId="Central Collector"
	/// /&gt;</code>
	/// <list type="bullet">
	///   <item>
	///     <term>connectionString</term>
	///     <description>f[^x[Xւ̐ڑɗpڑB</description>
    ///   </item>
    ///   <item>
    ///     <term>table</term>
    ///     <description>SyslogbZ[Wۑm̃XL[}ƌ݊̂e[uB</description>
    ///   </item>
    ///   <item>
    ///     <term>collectorId</term>
    ///     <description>f[^x[X̃e[uŊeRN^ʂIDB</description>
    ///   </item>
	/// </list>
	/// </example>
    /// <remarks>
    /// <para>L̂悤ȃXL[}f[^x[X e[u Syslog bZ[Wۑ܂B</para>
    /// <list type="table">
    ///   <listheader>
    ///     <term></term>
    ///     <description></description>
    ///   </listheader>
    ///   <item>
    ///     <term>RecievedTime</term>
    ///     <description>datetime ^B</description>
    ///   </item>
    ///   <item>
    ///     <term>CollectorId</term>
    ///     <description>nvarchar ^B ̃TCY 256 oCgB</description>
    ///   </item>
    ///   <item>
    ///     <term>LocalInterfaceAddress</term>
    ///     <description>nvarchar ^B ̃TCY 40 oCgB</description>
    ///   </item>
    ///   <item>
    ///     <term>LocalInterfacePort</term>
    ///     <description>int ^B</description>
    ///   </item>
    ///   <item>
    ///     <term>RemoteHostname</term>
    ///     <description>nvarchar ^B ̃TCY 256 oCgB</description>
    ///   </item>
    ///   <item>
    ///     <term>RemoteAddress</term>
    ///     <description>nvarchar ^B ̃TCY 40 oCgB</description>
    ///   </item>
    ///   <item>
    ///     <term>RemotePort</term>
    ///     <description>int ^B</description>
    ///   </item>
    ///   <item>
    ///     <term>Facility</term>
    ///     <description>int ^B</description>
    ///   </item>
    ///   <item>
    ///     <term>Severity</term>
    ///     <description>int ^B</description>
    ///   </item>
    ///   <item>
    ///     <term>ReceivedTimestamp</term>
    ///     <description>datetime ^B</description>
    ///   </item>
    ///   <item>
    ///     <term>Hostname</term>
    ///     <description>nvarchar ^B ̃TCY 64 oCgB</description>
    ///   </item>
    ///   <item>
    ///     <term>Tag</term>
    ///     <description>nvarchar ^B ̃TCY 32 oCgB</description>
    ///   </item>
    ///   <item>
    ///     <term>Content</term>
    ///     <description>nvarchar ^B ̃TCY 1024 oCgB</description>
    ///   </item>
    /// </list>
    /// <example>Syslog bZ[Wۑpf[^x[X e[u쐬SQLXe[ggB(SQL Server)
    /// <code>CREATE TABLE [SyslogDetails]
    /// (
    ///   SyslogID              int             IDENTITY(1, 1),
    ///   ReceivedTime          datetime        NOT NULL,
    ///   CollectorId           nvarchar(256)   NOT NULL,
    ///   LocalInterfaceAddress nvarchar(40)    NOT NULL,
    ///   LocalInterfacePort    int             NOT NULL,
    ///   RemoteHostname        nvarchar(256)   NOT NULL,
    ///   RemoteAddress         nvarchar(40)    NOT NULL,
    ///   RemotePort            int             NOT NULL,
    ///   Facility              int             NOT NULL,
    ///   Severity              int             NOT NULL,
    ///   ReceivedTimestamp     datetime        NOT NULL,
    ///   Hostname              nvarchar(64)    NOT NULL,
    ///   Tag                   nvarchar(32)    NULL,
    ///   Content               nvarchar(1024)  NULL,
    ///   
    ///   CONSTRAINT pk_syslogid PRIMARY KEY NONCLUSTERED (SyslogID)
    /// )</code>
    /// </example>
	/// </remarks>
    #endregion
	public class SimpleDbWriter : ISyslogWriter
    {
        #region Config Key
        // App.config ̃JX^RtBO[Vǂݍݍۂɗp鑮
        private const string connectionStringKey = "connectionString";
        private const string databaseKey = "database";
        private const string ownerKey = "owner";
        private const string tableKey = "table";
        private const string collectorIdKey = "collectorId";
        #endregion

        // f[^x[XɃR[hǉ邽߂SQL
        private const string insertSql = 
            "INSERT INTO {0}.{1}.{2} " + 
                "(ReceivedTime, CollectorId, LocalInterfaceAddress, LocalInterfacePort," +
                " RemoteHostname, RemoteAddress, RemotePort," +
                " Facility, Severity, ReceivedTimestamp, Hostname, Tag, Content) " +
            "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

        private string connectionString = String.Empty;
        private string database = String.Empty;
        private string owner = "dbo";
        private string table = "SyslogDetails";
        private string collectorId = String.Empty;

        private IDictionary ConfigSetting = new Hashtable();

        /// <summary>
        /// RXgN^BApp.config XMLGg󂯎JX^RtBO[V擾܂B
        /// </summary>
        /// <param name="setting">App.config Ŏw肳ꂽǉݒm[h</param>
        public SimpleDbWriter(string setting)
        {
            if (setting != null)
            {
                // App.config ̃f[^瑮̃fBNVi擾
                ConfigSetting = (IDictionary) CustomValueSectionHandler.Parse(setting);
            }

            if (ConfigSetting.Contains(connectionStringKey))
            {
                connectionString = (string) ConfigSetting[connectionStringKey];
            }

            if (ConfigSetting.Contains(databaseKey))
            {
                database = (string) ConfigSetting[databaseKey];
                database = database.Replace("'", "''").Trim();
            }

            if (ConfigSetting.Contains(ownerKey))
            {
                owner = (string) ConfigSetting[ownerKey];
                owner = owner.Replace("'", "''").Trim();
            }

            if (ConfigSetting.Contains(tableKey))
            {
                table = (string) ConfigSetting[tableKey];
                table = table.Replace("'", "''").Trim();
            }

            if (ConfigSetting.Contains(collectorIdKey))
            {
                collectorId = (string) ConfigSetting[collectorIdKey];
                collectorId = collectorId.Replace("'", "''").Trim();

                if (collectorId == String.Empty)
                {
                    // RN^IDw肳ĂȂƂ̓[JzXggp
                    collectorId = Dns.GetHostName();
                }
            }
        }

        #region ISyslogWriter o
        /// <summary>
        /// f[^x[Xւ̐ڑ܂B
        /// </summary>
        public void Open()
        {
        }

        /// <summary>
        /// obt@Ăf[^f[^x[X֏݂܂B
        /// </summary>
        public void Flush()
        {
        }

        /// <summary>
        /// f[^x[Xւ̐ڑ\[XJ܂B
        /// </summary>
        public void Close()
        {
            this.Dispose();
        }

        /// <summary>
        /// SyslogInfo NX̃f[^f[^x[Xɕۑ܂B
        /// </summary>
        /// <param name="message">Syslog pPbg̃f[^JvZ SyslogInfoB</param>
        public void WriteSyslog(SyslogInfo message)
        {
            // using Xe[gg Dispose() mɌĂяo
            using (OleDbConnection conn = new OleDbConnection())
            {
                using (OleDbCommand cmd = this.CreateParameterCommand(conn))
                {
                    SyslogMessage msg = message.Message;

                    try
                    {
                        conn.ConnectionString = connectionString;
                        conn.Open();

                        // p[^RNV擾
                        OleDbParameterCollection paramColl = cmd.Parameters;

                        paramColl["ReceivedTime"].Value = message.ReceivedDateTime;
                        paramColl["CollectorId"].Value = collectorId;
                        paramColl["LocalInterfaceAddress"].Value = message.LocalEndPoint.Address.ToString();
                        paramColl["LocalInterfacePort"].Value = message.LocalEndPoint.Port;
                        paramColl["RemoteHostname"].Value = Dns.GetHostByAddress(message.RemoteEndPoint.Address).HostName;
                        paramColl["RemoteAddress"].Value = message.RemoteEndPoint.Address.ToString();
                        paramColl["RemotePort"].Value = message.RemoteEndPoint.Port;
                        paramColl["Facility"].Value = msg.Facility;
                        paramColl["Severity"].Value = msg.Severity;
                        paramColl["ReceivedTimestamp"].Value = msg.TimeStamp;
                        paramColl["Hostname"].Value = msg.Hostname;
                        paramColl["Tag"].Value = msg.TAG;
                        paramColl["Content"].Value = msg.CONTENT;

                        // NGs
                        cmd.ExecuteNonQuery();
                    }
                    catch (OleDbException e)
                    {
                        System.Diagnostics.Debug.WriteLine(e);
                    }
                }
            }
        }

        /// <summary>
        /// Syslog Writer̃thԂ܂B
        /// </summary>
        public string Name
        {
            get
            {
                return "Syslog Simple OleDb Database Writer";
            }
        }
        #endregion

        #region IDisposable o
        /// <summary>
        /// NXێĂA}l[W\[XJ܂B
        /// </summary>
        public void Dispose()
        {
        }
        #endregion

        /// <summary>
        /// p[^ꂽNG𗘗p邽߂ OleDbCommand 쐬܂B
        /// </summary>
        /// <param name="conn">ƂȂڑ\ OleDbConnectionB</param>
        /// <returns>p[^ꂽ OleDbCommandB</returns>
        private OleDbCommand CreateParameterCommand(OleDbConnection conn)
        {
            OleDbCommand cmd = conn.CreateCommand();

            cmd.CommandType = CommandType.Text;
            cmd.CommandText = String.Format(insertSql, database, owner, table);

            OleDbParameterCollection paramColl = cmd.Parameters;
            paramColl.Add("ReceivedTime", OleDbType.Date);
            paramColl.Add("CollectorId", OleDbType.VarWChar, 256);
            paramColl.Add("LocalInterfaceAddress", OleDbType.VarWChar, 40);
            paramColl.Add("LocalInterfacePort", OleDbType.Integer);
            paramColl.Add("RemoteHostname", OleDbType.VarWChar, 256);
            paramColl.Add("RemoteAddress", OleDbType.VarWChar, 40);
            paramColl.Add("RemotePort", OleDbType.Integer);
            paramColl.Add("Facility", OleDbType.Integer);
            paramColl.Add("Severity", OleDbType.Integer);
            paramColl.Add("ReceivedTimestamp", OleDbType.Date);
            paramColl.Add("Hostname", OleDbType.VarWChar, 64);
            paramColl.Add("Tag", OleDbType.VarWChar, 32);
            paramColl.Add("Content", OleDbType.VarWChar, 1024);

            cmd.Prepare();

            return cmd;
        }
    }
}
