using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Text;
using System.Xml;
using Syslog.Config.SectionHandler;

namespace Syslog.Collector.Writer
{
	/// <summary>
	/// Syslog Relay Controller 烋[eBOꂽ <see cref="SyslogInfo" /> CxgOɕۑ܂B
	/// </summary>
	/// <example>
	/// <para>App.config ݒB</para>
    /// <code>&lt;eventlog machine="Rs[^" format="text|xml" /&gt;</code>
    /// <list type="bullet">
    ///   <item>
    ///     <term>machine</term>
    ///     <description>CxgOL^ڑRs[^B[JzXgɋL^ꍇ "."B</description>
    ///   </item>
    ///   <item>
    ///     <term>format</term>
    ///     <description>CxgOɋL^tH[}bg`BText ` XML `włB</description>
    ///   </item>
    /// </list>
    /// </example>
	public class SimpleEventLogWriter : ISyslogWriter
    {
        // App.config JX^ݒǂݍލۂɗp鑮
        private const string machineKey = "machine";

        // bZ[WۑCxgO̖O
        private const string logName = "Syslog";
        // CxgOɏނ̃Rg[̃\[X
        private const string sourceName = "Managed Syslog Server";
        // CxgOɕۑtf[^̃tH[}bg` - yyyy-MM-ddTHH:mm:ss
        private const string datetimeFormat = "yyyy-MM-ddTHH:mm:ss";

        private IDictionary ConfigSetting = new Hashtable();
        private EventLog eventLog = null;
        // CxgOۑRs[^
        private string machineName = ".";

        /// <summary>
        /// Kvȃ\[XJfXgN^B
        /// </summary>
        ~SimpleEventLogWriter()
        {
            try
            {
                this.Dispose();
            }
            catch
            {
            }
        }

        /// <summary>
        /// App.config Ɏw肳ꂽJX^RtBO[V󂯎ďRXgN^B
        /// </summary>
        /// <param name="setting">App.config Ɏw肳ꂽJX^RtBO[Vm[hB</param>
        public SimpleEventLogWriter(string setting)
        {
            if (setting != null)
            {
                ConfigSetting = (IDictionary) CustomValueSectionHandler.Parse(setting);
            }

            if (ConfigSetting.Contains(machineKey))
            {
                // CxgOۑRs[^
                machineName = (string) ConfigSetting[machineKey];
            }
        }

        /// <summary>
        /// [JzXgɃJX^CxgO쐬܂
        /// </summary>
        internal static void CreateEventSource()
        {
            try
            {
                // Cxgݐ悪w肳Ăꍇ
                if (EventLog.SourceExists(sourceName))
                {
                    // Cxgݐx폜
                    EventLog.DeleteEventSource(sourceName);
                }

                // Cxg\[X쐬
                EventLog.CreateEventSource(sourceName, logName);
            }
            catch (Exception e)
            {
                Debug.Assert(false, "CxgÕWXgGgɃANZXł܂ł", e.ToString());

                throw e;
            }
        }

        /// <summary>
        /// [JzXg̃JX^CxgO폜܂
        /// </summary>
        internal static void DeleteEventSource()
        {
            // JX^CxgO݂ꍇ
            if (EventLog.Exists(logName))
            {
                // JX^CxgO폜
                EventLog.Delete(logName);
            }
        }

        /// <summary>
        /// CxgOɏނ̃Rg[̃\[XԂ܂
        /// </summary>
        internal static string Source
        {
            get
            {
                return sourceName;
            }
        }

        /// <summary>
        /// bZ[WۑCxgO̖OԂ܂
        /// </summary>
        internal static string Log
        {
            get
            {
                return logName;
            }
        }

        #region ISyslogWriter o
        /// <summary>
        /// CxgOւ݂̏܂B
        /// </summary>
        public void Open()
        {
            // CxgOIuWFNg̃CX^Xłɍ쐬Ăꍇ
            if (eventLog != null)
            {
                // CX^X̃nhN[Y
                eventLog.Dispose();
            }

            try
            {
                // CxgOɃANZXĂ݂
                EventLog.Exists(logName, machineName);
            }
            catch (ArgumentException e)
            {
                // w肵[gRs[^݂Ȃꍇ
                if (e.ParamName == "machineName")
                {
                    Debug.Assert(false, "w肳ꂽ[gRs[^݂܂", e.ToString());

                    Trace.WriteLine("w肳ꂽ[gRs[^݂܂B[JRs[^̃CxgO𗘗p܂B", "Syslog.Collector.Writer.SimpleEventLogWriter");

                    // O̕ۑ[JRs[^ɐݒ
                    machineName = ".";
                }
                else
                {
                    // OX[
                    throw e;
                }
            }

            try
            {
                // Cxgݐ悪w肳ĂȂꍇ
                if (!EventLog.SourceExists(sourceName, machineName))
                {
                    // Cxg\[X쐬
                    EventLog.CreateEventSource(sourceName, logName, machineName);
                }
            }
            catch (System.Security.SecurityException e)
            {
                Debug.Assert(false, "CxgÕWXgGgɃANZXł܂ł", e.ToString());

                Trace.WriteLine("CxgÕWXgGgɃANZXł܂ł", "Syslog.Collector.Writer.SimpleEventLogWriter");
                Trace.Indent();
                Trace.WriteLine(e, "Syslog.Collector.Writer.SimpleEventLogWriter");
                Trace.Unindent();

                throw e;
            }

            // CxgOIuWFNg쐬
            eventLog = new EventLog();
            // Cxg\[Xw肷
            eventLog.Source = sourceName;

            // NbZ[WL^
            eventLog.WriteEntry(String.Format("The {0} was started.", this.Name), EventLogEntryType.Information);
        }

        /// <summary>
        /// obt@ɂf[^𑦎݂܂B
        /// </summary>
        public void Flush()
        {
            return;
        }

        /// <summary>
        /// CxgOւ̐ڑă\[XJ܂B
        /// </summary>
        public void Close()
        {
            this.Dispose();
        }

        /// <summary>
        /// SyslogInfo CxgOɏ݂܂B
        /// </summary>
        /// <param name="message">Syslog Relay Controller 烋[eBOꂽ SyslogInfoB</param>
        public void WriteSyslog(SyslogInfo message)
        {
            SyslogMessage msg = message.Message;

            // CxgOɃOL^, EventID = Dx, Category = @\
            eventLog.WriteEntry(this.Transform(message), this.GetEntryType(msg.Severity), msg.Priority, (short) msg.Facility);
        }

        /// <summary>
        /// SyslogWriter ̃thԂ܂B
        /// </summary>
        public string Name
        {
            get
            {
                return "Syslog Simple Windows Eventlog Writer";
            }
        }
        #endregion

        #region IDisposable o
        /// <summary>
        /// CX^XێĂ郊\[X܂B
        /// </summary>
        public void Dispose()
        {
            if (eventLog != null)
            {
                try
                {
                    // ~bZ[WL^
                    eventLog.WriteEntry(String.Format("The {0} was stopped.", this.Name), EventLogEntryType.Information);
                }
                catch
                {
                }
                finally
                {
                    // CxgON[Y
                    eventLog.Dispose();
                }

                // CxgOւ̎QƂ null ɐݒ肷
                eventLog = null;
            }
        }
        #endregion

        /// <summary>
        /// Syslog  Severity lCxgÕCxgJeSɃ}bsO܂
        /// </summary>
        /// <param name="severity">Severity R[hlB</param>
        /// <returns>CxgÕGg^CvB</returns>
        protected virtual EventLogEntryType GetEntryType(int severity)
        {
            switch (severity)
            {
                case 7: // Debug
                case 6: // Informational
                case 5: // Notice
                {
                    return EventLogEntryType.Information;
                }
                case 4: // Warning
                {
                    return EventLogEntryType.Warning;
                }
                default: // Error
                {
                    return EventLogEntryType.Error;
                }
            }
        }

        /// <summary>
        /// App.config ɂĎw肳ꂽCxgLqqtH[}bg SyslogInfo ϊ܂B
        /// </summary>
        /// <param name="info">SyslogbZ[W̃f[^JvZSyslogInfoB</param>
        /// <returns></returns>
        protected virtual string Transform(SyslogInfo info)
        {
            // tH[}bg̎w擾
            string format = (string) ConfigSetting["format"];

            // tH[}bgw肳Ă xml w肳Ăꍇ
            if (format != null && format.ToLower() == "xml")
            {
                // XML`ɃtH[}bg
                return this.SyslogToXml(info);
            }
            // tH[}bgw肪sȏꍇ
            else
            {
                // eLXg`ɃtH[}bg
                return this.SyslogToString(info);
            }
        }

        /// <summary>
        /// SyslogInfo eLXg`ɃtH[}bg܂B
        /// </summary>
        /// <remarks><example>
        /// <para>tH[}bgeLXg`̃Tv</para>
        /// <code>Priority: 	13, user-level messages, Notice
        /// Timestamp: 	2003/11/21 1:30:03.123
        /// Hostname: 	router, fe80::1, 514
        /// Interface: 	::, 514
        /// Content: 	PP[01] Rejected at IN(200021) filter: TCP 192.168.0.1:1234 > 172.26.0.1:445
        /// Option: 	
        /// 
        /// </code>
        /// </example></remarks>
        /// <param name="info">SyslogbZ[Wf[^܂ SyslogInfoB</param>
        /// <returns>eLXg`ɃtH[}bgꂽSyslogbZ[WB</returns>
        protected virtual string SyslogToString(SyslogInfo info)
        {
            // f[^؂Zp[^
            const string csv = ", ";

            // obt@ 1536 ܂Ŋmۂ
            StringBuilder sb = new StringBuilder(1536);
            StringWriter sw = new StringWriter(sb);

            SyslogMessage msg = info.Message;

            // Dx
            sw.Write("Priority: \t");
            sw.Write(msg.Priority);
            sw.Write(csv);
            // Facility ̕\L
            sw.Write(Priority.ToString((Facility) msg.Facility, false));
            sw.Write(csv);
            // Severity ̕\L
            sw.WriteLine(Priority.ToString((Severity) msg.Severity, true));

            // Cxg - yyyy-MM-ddTHH:mm:ss
            sw.Write("Timestamp: \t");
            sw.WriteLine(msg.TimeStamp.ToString(datetimeFormat));

            // CxgzXg
            sw.Write("Hostname: \t");
            sw.Write(msg.Hostname);
            sw.Write(csv);
            // \[XIPAhX
            sw.Write(info.RemoteEndPoint.Address.ToString());
            sw.Write(csv);
            // \[X|[g
            sw.WriteLine(info.RemoteEndPoint.Port);

            // MC^[tFCXAhX
            sw.Write("Interface: \t");
            sw.Write(info.LocalEndPoint.Address.ToString());
            sw.Write(csv);
            // M|[g
            sw.WriteLine(info.LocalEndPoint.Port);

            // Tag  Content 
            sw.Write("Content: \t");
            sw.WriteLine(msg.MSG);

            // ݒ肳ĂIvVn
            sw.Write("Option: \t");

            // ׂẴIvVo
            foreach (object obj in info.OptionList.Keys)
            {
                sw.Write(obj.ToString());
                sw.Write(csv);

                if (info.OptionList.Contains(obj))
                {
                    sw.WriteLine(info.OptionList[obj].ToString());
                }

                sw.Write("\t\t");
            }

            // so
            sw.WriteLine();
            // StringWriter 
            sw.Close();

            // f[^̏܂ꂽ StringBuilder 當Ԃ
            return sb.ToString();
        }

        /// <summary>
        /// SyslogInfo XML`ɃtH[}bg܂B
        /// </summary>
        /// <remarks><example>
        /// <para>tH[}bgXML`̃Tv</para>
        /// <code>
        /// <para>&lt;?xml version="1.0" encoding="utf-16"?&gt;</para>
        /// <para>&lt;syslogEntry priority="13" facility="1" severity="5" timestamp="2003-11-22T04:00:40" hostname="router" remoteHost="172.26.0.1" remotePort="514" localHost="0.0.0.0" localPort="514" content="Rejected directed broadcast: ICMP 192.168.0.1 &gt; 172.26.0.1 : echo request" /&gt;</para>
        /// </code>
        /// </example></remarks>
        /// <param name="info">SyslogbZ[Wf[^܂ SyslogInfoB</param>
        /// <returns>XML`ɃtH[}bgꂽSyslogbZ[WB</returns>
        protected virtual string SyslogToXml(SyslogInfo info)
        {
            // ̃obt@ 2048 ܂ŗpӂ
            StringBuilder sb = new StringBuilder(2048);
            StringWriter sw = new StringWriter(sb);
            XmlTextWriter xw = new XmlTextWriter(sw);

            SyslogMessage msg = info.Message;

            // CfgLɂ
            xw.Formatting = Formatting.Indented;

            // XML錾
            xw.WriteStartDocument();
            // [gGg
            xw.WriteStartElement("syslogEntry");

            // Priority 
            xw.WriteStartAttribute("priority", null);
            xw.WriteString(msg.Priority.ToString());
            xw.WriteEndAttribute();

            // Facility 
            xw.WriteStartAttribute("facility", null);
            xw.WriteString(msg.Facility.ToString());
            xw.WriteEndAttribute();

            // Severity 
            xw.WriteStartAttribute("severity", null);
            xw.WriteString(msg.Severity.ToString());
            xw.WriteEndAttribute();

            // Timestamp 
            xw.WriteStartAttribute("timestamp", null);
            xw.WriteString(msg.TimeStamp.ToString(datetimeFormat));
            xw.WriteEndAttribute();

            // CxgzXg 
            xw.WriteStartAttribute("hostname", null);
            xw.WriteString(msg.Hostname);
            xw.WriteEndAttribute();

            // \[XIPAhX 
            xw.WriteStartAttribute("remoteHost", null);
            xw.WriteString(info.RemoteEndPoint.Address.ToString());
            xw.WriteEndAttribute();

            // \[X|[g
            xw.WriteStartAttribute("remotePort", null);
            xw.WriteString(info.RemoteEndPoint.Port.ToString());
            xw.WriteEndAttribute();

            // [JC^[tFCXIPAhX 
            xw.WriteStartAttribute("localHost", null);
            xw.WriteString(info.LocalEndPoint.Address.ToString());
            xw.WriteEndAttribute();

            // [J|[g 
            xw.WriteStartAttribute("localPort", null);
            xw.WriteString(info.LocalEndPoint.Port.ToString());
            xw.WriteEndAttribute();

            // TagContent
            xw.WriteStartAttribute("content", null);
            xw.WriteString(msg.MSG);
            xw.WriteEndAttribute();
            
            // Option Gg쐬
            foreach (object obj in info.OptionList.Keys)
            {
                // IvV `ChGg쐬
                xw.WriteStartElement("option", null);

                // Key 쐬
                xw.WriteStartAttribute("key", null);
                // IvṼL[l
                xw.WriteString(obj.ToString());
                xw.WriteEndAttribute();

                // L[lɑΉf[^ꍇ
                if (info.OptionList.Contains(obj))
                {
                    // IvṼf[^
                    xw.WriteString(info.OptionList[obj].ToString());
                }
                // L[lɑΉf[^Ȃꍇ
                else
                {
                    // 󕶎eLXgm[hƂ
                    xw.WriteString(String.Empty);
                }
                
                xw.WriteEndElement();
            }

            // [gGg
            xw.WriteEndElement();
            // XMLm[h
            xw.WriteEndDocument();
            // XmlTextWriter 
            xw.Close();
            // ƂȂ StringWriter 
            sw.Close();

            // f[^̏܂ꂽ StringBuilder 當Ԃ
            return sb.ToString();
        }
    }
}
