using System;
using System.Collections;
using System.Configuration;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using Syslog.Config.SectionHandler;

namespace Syslog.Collector.Writer
{
    #region NX
	/// <summary>
	/// Syslog Relay Controller 󂯎 <see cref="SyslogInfo" /> Ot@Cɕۑ܂B
	/// </summary>
	/// <example>
	/// <para>App.config ݒB</para>
	/// <code>
	/// &lt;simpleFileWriter
	///  rotation="daily|weekly|monthly"
	///  outputPath="./Logfiles/Syslog"
	///  encoding="utf-16"&gt;
	///     {DateTime} {DestAddress:host} {Syslog:PRI}{Syslog:HEADER} {Syslog:MSG}
	/// &lt;/simpleFileWriter&gt;</code>
	/// <list type="bullet">
	///   <item>
	///     <term>rotation</term>
	///     <description>Ot@C̃[e[VԊuBDaily, Weekly, Monthly p\B</description>
    ///   </item>
    ///   <item>
    ///     <term>outputPath</term>
    ///     <description>Ot@CۑpXBw肳ꂽpXɎIɃOt@C܂B</description>
    ///   </item>
    ///   <item>
    ///     <term>encoding</term>
    ///     <description>Ot@C̕R[hB</description>
    ///   </item>
    ///   <item>
    ///     <term>eLXg m[h</term>
    ///     <description>SyslogbZ[Wƒu鐳K\p^[</description>
    ///   </item>
	/// </list>
	/// <para>eLXgm[hɎwł鐳K\̃p^[B</para>
	/// <list type="bullet">
	///   <listheader>
	///     <term>K\p^[</term>
	///     <description>uSyslogbZ[W̓eB</description>
    ///   </listheader>
    ///   <item>
    ///     <term>{Syslog}</term>
    ///     <description>SyslogpPbg̃f[^ɒu܂B
    ///         <para>{Syslog:<c>&lt;part&gt;</c>}  <c>part</c> w肷邱ƂɂASyslogbZ[W̃p[goĒu邱Ƃ\łB</para>
    ///         <para><c>part</c>  <c>PRI</c>, <c>HEADER</c>, <c>MSG</c> w肷SyslogbZ[W炻ꂼ̃p[gA
    ///         <c>Priority</c> ͗DxA <c>Facility</c> ͋@\A <c>Severity</c> ͏dxɁA
    ///         <c>TimeStamp</c> SyslogbZ[W̃^CX^vA <c>HostName</c> SyslogbZ[W̃zXgA
    ///         <c>Tag</c> TAGɁA <c>Content</c> CONTENTɒus܂B</para>
    ///     </description>
    ///   </item>
	///   <item>
	///     <term>{DateTime}</term>
	///     <description>DateTime.ToString("s") ɑ ISO 8601tH[}bg (yyyy-mm-ddThh:nn:ss) ̌ݎɒu܂B
	///         <para>{DateTime:<c>&lt;format&gt;</c>}  <c>format</c> ɓtw肷邱ƂɂA̓ttH[}bgւ̒u\łB</para>
	///     </description>
    ///   </item>
	///   <item>
	///     <term>{DestinationAddress}</term>
	///     <description>SyslogpPbgMC^[tFCX̃AhXB
	///         <para>{DestAddr:<c>&lt;ipstyle&gt;</c>}  <c>ipstyle</c>  <c>port</c> w肷ƃ|[gԍA
	///         <c>host</c> w肷IPAhXtzXgAallw肷IPAhXƃ|[gԍɒu܂B</para>
	///     </description>
	///   </item>
    ///   <item>
    ///     <term>{SourceAddresss}</term>
    ///     <description>SyslogpPbg̑MAhXB
	///         <para>{SrcAddr:<c>&lt;ipstyle&gt;</c>}  <c>ipstyle</c>  <c>port</c> w肷ƃ|[gԍA
	///         <c>host</c> w肷IPAhXtzXgAallw肷IPAhXƃ|[gԍɒu܂B</para>
	///     </description>
    ///   </item>
    ///   <item>
    ///     <term>{Option:<c>&lt;optionKey&gt;</c>}</term>
    ///     <description>tB^ȂǂɂĒǉꂽIvVlɒu܂Bw肳ꂽIvVl݂Ȃꍇ́A󔒕ƒu܂B</description>
    ///   </item>
	/// </list>
	/// </example>
    #endregion
	public class SimpleFileWriter : ISyslogWriter
	{
        // ftHg̃OۑpX
        private const string OutputLogFilePath = @"%SystemRoot%\System32\Logfiles\Syslog";
        private const string ConfigReturnCodeKey = "replaceReturnCode";

        private StreamWriter SyslogWriter = null;
        private IDictionary ConfigSetting = new Hashtable();
        private FileInfo OutputFile = null;
        private Regex ReplaceReturnCode = new Regex(@"\r\n|\n", RegexOptions.Compiled | RegexOptions.CultureInvariant | RegexOptions.ExplicitCapture | RegexOptions.IgnoreCase);
        private string ReturnCode = Environment.NewLine;
        private bool disposed = false;

        /// <summary>
        /// App.config ̃JX^RtBO[Vm[h󂯎RXgN^B
        /// </summary>
        /// <param name="setting">App.config Ŏw肳ꂽJX^RtBO[Ṽm[hB</param>
        public SimpleFileWriter(string setting)
        {
            if (setting != null)
            {
                ConfigSetting = (IDictionary) CustomValueSectionHandler.Parse(setting);

                this.GetReplacedReturnCode();
            }
        }

        #region ISyslogWriter o

        /// <summary>
        /// Ot@CJ܂B
        /// </summary>
        public void Open()
        {
            // łɊJĂt@Cꍇ
            if (SyslogWriter != null)
            {
                // t@CN[Y܂
                SyslogWriter.Close();

                // \[Xēxgp邱ƂL^
                this.disposed = false;
            }

            // eLXgC^
            this.SetupTextWriter(DateTime.Now);

            // C^̋No܂
            SyslogWriter.WriteLine("# {0} - {1} started.", DateTime.Now.ToString("s"), this.Name);
        }

        /// <summary>
        /// obt@Ă郍Ot@Cɏo܂B
        /// </summary>
        public void Flush()
        {
            // C^Ăꍇ
            if (SyslogWriter != null)
            {
                // obt@tbV
                SyslogWriter.Flush();
            }
        }

        /// <summary>
        /// obt@̃f[^t@Cɏo\[XJ܂B
        /// </summary>
        /// <remarks>
        /// Close() ĂяoȂƃf[^ꍇ܂B
        /// IɕKĂяoKv܂B
        /// </remarks>
        public void Close()
        {
            this.Dispose();
        }

        /// <summary>
        /// SyslogInfo Ot@Cɏo܂B
        /// </summary>
        /// <param name="message">Syslog Relay Controller 烋[eBOꂽ SyslogInfoB</param>
        /// <exception cref="ArgumentNullException">SyslogInfo @nullQ łB</exception>
        /// <exception cref="ObjectDisposedException"></exception>
        public void WriteSyslog(SyslogInfo message)
        {
            if (message == null)
            {
                throw new ArgumentNullException("message");
            }

            // t@CC^Ă邱ƂmF
            if (SyslogWriter != null)
            {
                // Ot@CpPbgMɂčēx肷
                FileInfo OutputPath = this.GetOutputPath(message.ReceivedDateTime);

                // Ot@C̃tpXύXĂꍇ
                if (OutputFile.FullName != OutputPath.FullName)
                {
                    // C^
                    SyslogWriter.Close();

                    // pPbgMɃeLXgC^Ȃ
                    this.SetupTextWriter(message.ReceivedDateTime);
                }

                // OɃf[^o
                SyslogWriter.WriteLine(this.GetFormattedMessage(message));
            }
            else
            {
                // ̃NX Dispose() ͂łɌĂяoĂꍇ
                throw new ObjectDisposedException("SyslogWriter");
            }
        }

        /// <summary>
        /// Syslog Writer ̃thłB
        /// </summary>
        public string Name
        {
            get
            {
                return "Syslog Simple File Writer";
            }
        }
        #endregion

        /// <summary>
        /// f[^ۑׂt@C肵܂B
        /// </summary>
        /// <param name="date">t@C肷ۂɎQƂB</param>
        /// <returns>f[^ۑׂt@C\ FileInfoB</returns>
        protected virtual FileInfo GetOutputPath(DateTime date)
        {
            IDictionary config = ConfigSetting;

            // t@C̃[e[VԊu
            string rotation = (string) config["rotation"];
            // t@Ci[pX
            string outputPath = (string) config["outputPath"];
            // pXۂ̃t@C
            string FileName = null;
            // J`ɈˑȂtH[}bg
            DateTimeFormatInfo DateInfo = DateTimeFormatInfo.InvariantInfo;

            if (rotation == null)
            {
                rotation = String.Empty;
            }

            if (outputPath == null)
            {
                outputPath = String.Empty;
            }

            #region t@C胋[`
            switch (rotation.ToLower())
            {
                // TPʂŐVt@C쐬
                case "weekly":
                {
                    // ߂vZ
                    DateTime FirstDay = date.AddDays(1 + -date.Day);
                    // ߂̏Tԍ擾
                    int FirstWeek = DateInfo.Calendar.GetWeekOfYear(FirstDay, DateInfo.CalendarWeekRule, DateInfo.FirstDayOfWeek);
                    // ̏Tԍ擾
                    int Week = DateInfo.Calendar.GetWeekOfYear(date, DateInfo.CalendarWeekRule, DateInfo.FirstDayOfWeek);
                    // ̏Tԍ猎߂̏Tԍ - 1Z
                    Week -= --FirstWeek;

                    // t@C肷 - syslog{yyyyMM}{Tԍ}.log
                    FileName = String.Format("syslog{0}{1}.log", DateTime.Now.ToString("yyyyMM", DateInfo), Week);

                    break;
                }
                // PʂŐVt@C쐬
                case "monthly":
                {
                    // t@C肷 - syslog{yyyyMM}.log
                    FileName = String.Format("syslog{0}.log", DateTime.Now.ToString("yyyyMM", DateInfo));

                    break;
                }
                // Vt@C쐬
                default:
                {
                    // t@C肷 - syslog{yyyyMMdd}.log
                    FileName = String.Format("syslog{0}.log", DateTime.Now.ToString("yyyyMMdd", DateInfo));

                    break;
                }
            }
            #endregion

            // t@CۑpXw肳ĂȂꍇ
            if (outputPath == String.Empty)
            {
                // ݂̃tH_ftHgOt@CtH_ɃpXݒ
                outputPath = OutputLogFilePath;
            }

            // pXƃt@CAϐWJ FileInfo NX쐬
            FileInfo info = new FileInfo(System.Environment.ExpandEnvironmentVariables(Path.Combine(outputPath, FileName)));

            try
            {
                // KvȕۑtH_쐬
                info.Directory.Create();
            }
            catch (UnauthorizedAccessException e)
            {
                Debug.Assert(false, "Ot@C̕ۑfBNg쐬ł܂łB", e.ToString());

                Trace.WriteLine("Ot@C̕ۑfBNg쐬ł܂ł!", "Syslog.Collector.Writer.SimpleFileWriter");

                throw e;
            }

            return info;
        }

        /// <summary>
        /// App.config ̃JX^RtBO[V eLXgm[hSyslogbZ[W̃tH[}bg肵܂
        /// </summary>
        /// <param name="info">App.config Ŏw肳ꂽK\eLXgm[h</param>
        /// <returns>w肳ꂽ`ɃtH[}bgꂽ</returns>
        protected virtual string GetFormattedMessage(SyslogInfo info)
        {
            // eLXgm[h̐K\擾
            string Format = (string) ConfigSetting[String.Empty];

            // bZ[W{Ɋ܂܂sR[h(LF/CR-LF)u
            info.Message.SetMSG(ReplaceReturnCode.Replace(info.Message.MSG, ReturnCode));

            // SyslogFormatter  SyslogInfo eLXg`ɃVACY
            return SyslogFormatter.Serialize(info, Format);
        }

        #region IDisposable o

        /// <summary>
        /// A}l[W\[XJ܂
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
        }
        #endregion

        /// <summary>
        /// A}l[W\[XJ܂
        /// </summary>
        /// <param name="disposing">ǉIɃ}l[W\[XJ邩肵܂</param>
        protected void Dispose(bool disposing)
        {
            // \[XgpĂꍇ
            if (!this.disposed)
            {
                // }l[W\[XJ
                if (disposing)
                {
                    if (SyslogWriter != null)
                    {
                        // C^̒~o܂
                        SyslogWriter.WriteLine("# {0} - {1} stoped.", DateTime.Now.ToString("s"), this.Name);

                        SyslogWriter.Close();

                        SyslogWriter = null;
                    }
                }
            }

            // \[XJƂL^
            this.disposed = true;
        }

        /// <summary>
        /// t@Cɏ StreamWriter ܂
        /// </summary>
        /// <param name="receivedTime">SyslogbZ[WMB</param>
        private void SetupTextWriter(DateTime receivedTime)
        {
            Encoding encoding = null;

            try
            {
                // GR[fBOݒ擾
                string encodingName = (string) ConfigSetting["encoding"];

                if (encodingName != null || encodingName != String.Empty)
                {
                    // GR[fBOݒ肷
                    encoding = Encoding.GetEncoding(encodingName);
                }
                else
                {
                    // ftHgUTF16ɐݒ肷
                    encoding = Encoding.Unicode;
                }
            }
            catch
            {
                // ftHgUTF16ɐݒ肷
                encoding = Encoding.Unicode;
            }

            // MɊÂăt@C
            OutputFile = this.GetOutputPath(receivedTime);

            try
            {
                // StreamWriter 쐬܂
                SyslogWriter = new StreamWriter(OutputFile.FullName, true, encoding);
            }
            catch (UnauthorizedAccessException e)
            {
                Debug.Assert(false, "Ot@Cւ̃ANZXۂw肳ꂽpXɃt@C쐬ł܂łB", e.ToString());

                Trace.WriteLine("Ot@COt@Cւ̃ANZXۂw肳ꂽpXɃt@C쐬ł܂ł!", "Syslog.Collector.Writer.SimpleFileWriter");
                Trace.Indent();
                Trace.WriteLine(e, "Syslog.Collector.Writer.SimpleFileWriter");
                Trace.Unindent();

                throw e;
            }

            SyslogWriter.AutoFlush = true;
        }

        /// <summary>
        /// bZ[W{̉sR[hu镶擾܂
        /// </summary>
        private void GetReplacedReturnCode()
        {

            // 񂪎w肳Ăꍇ
            if (ConfigSetting[ConfigReturnCodeKey] != null)
            {
                // u镶擾
                string ReplaceCode = ConfigSetting[ConfigReturnCodeKey].ToString();

                // GXP[v
                StringBuilder sb = new StringBuilder(ReplaceCode);
                sb.Replace(@"\\", "\\");
                sb.Replace(@"\0", "\0");
                sb.Replace(@"\a", "\a");
                sb.Replace(@"\b", "\b");
                sb.Replace(@"\f", "\f");
                sb.Replace(@"\n", "\n");
                sb.Replace(@"\r", "\r");
                sb.Replace(@"\t", "\t");
                sb.Replace(@"\v", "\v");

                // uۑ
                ReturnCode = sb.ToString();
            }
        }
    }
}
