using System;
using System.Collections;
using System.Configuration;
using System.Diagnostics;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Xml;

namespace Syslog.Collector.Writer
{
    /// <summary>
    /// Syslog Relay Controller 烋[eBOꂽ <see cref="SyslogInfo" /> 𑼂 Syslog T[oɓ]܂B
    /// </summary>
    /// <example>
    /// <para>App.config ݒB</para>
    /// <code>&lt;forward encoding="euc-jp"
    ///     destinationAddress="syslog1,syslog2"
    ///     destinationPort="514"
    ///     sourceAddress="0.0.0.0"
    ///     sourcePort="51400" /&gt;</code>
    /// <list type="table">
    ///   <listheader>
    ///     <term></term>
    ///     <description></description>
    ///   </listheader>
    ///   <item>
    ///     <term>encoding</term>
    ///     <description>Syslog bZ[W]ۂɗp镶GR[fBOB</description>
    ///   </item>
    ///   <item>
    ///     <term>destinationAddress</term>
    ///     <description>Syslog bZ[W] Syslog T[oBJ}ŋ؂邱Ƃɂ蕡̃T[owł܂B</description>
    ///   </item>
    ///   <item>
    ///     <term>destinationPort</term>
    ///     <description>]̃|[gԍBȗꍇ̓|[g 514 ɌđM܂B</description>
    ///   </item>
    ///   <item>
    ///     <term>sourceAddress</term>
    ///     <description>pPbg𑗐Mۂɗp郍[JAhXBIɃ[JAhXIꍇ :: ܂ 0.0.0.0Bȗꍇ 0.0.0.0 p܂B</description>
    ///   </item>
    ///   <item>
    ///     <term>sourcePort</term>
    ///     <description>pPbg𑗐Mۂɗp郍[J|[gԍBȗꍇ̓|[g 51400 p܂B</description>
    ///   </item>
    ///   <item>
    ///     <term>sizeLimit</term>
    ///     <description>]pPbg̍őTCYݒ肵܂Bw肵TCY傫ȃbZ[W͎w肵TCYɐ؂l߂ē]܂Bȗꂽꍇ 1024 oCgɐݒ肳܂B</description>
    ///   </item>
    /// </list>
    /// </example>
	public class SimpleSyslogForwarder : ISyslogWriter
	{
        private const string DisplayName = "Simple Syslog Forwarder";
        private const string ConfigForwardKey = "forward";
        private const string ConfigDestAddressKey = "destinationAddress";
        private const string ConfigDestPortKey = "destinationPort";
        private const string ConfigSourceAddressKey = "sourceAddress";
        private const string ConfigSourcePortKey = "sourcePort";
        private const string ConfigEncodingKey = "encoding";
        private const string ConfigMaxForwardingSizeKey = "sizeLimit";
        private const string DefaultLocalAddress = "0.0.0.0";
        private const int DefaultSyslogPort = 514;
        private const int DefaultLocalPort = 51400;
        private const int DefaultMaxForwardingSize = 1024;

        private UdpClient Forward = null;
        private string DestAddress = String.Empty;
        private int DestPort = DefaultSyslogPort;
        private IPEndPoint LocalEP = null;
        private Encoding encode = Encoding.GetEncoding("euc-jp");
        private int MaxForwardingSize = DefaultMaxForwardingSize;

        /// <summary>
        /// App.config Ɏw肳ꂽJX^RtBO[V󂯎ďRXgN^B
        /// </summary>
        /// <param name="setting">App.config Ɏw肳ꂽJX^RtBO[Vm[hB</param>
		public SimpleSyslogForwarder(string setting)
        {
            // [JAhXftHgɐݒ肷
            string LocalAddress = DefaultLocalAddress;
            // [J|[gftHgɐݒ肷
            int LocalPort = DefaultLocalPort;

            // JX^ݒǂݍ
            Hashtable config = this.GetCustomSettingNode(setting);

            // JX^ݒ肪ꍇ
            if (config != null)
            {
                // ڑzXgw肳ĂȂꍇ
                if (!config.Contains(ConfigDestAddressKey))
                {
                    // G[𔭐
                    throw new ArgumentException("]zXgw肳Ă܂", "DestAddress");
                }
                else
                {
                    // ڑzXgǂݎ
                    DestAddress = config[ConfigDestAddressKey].ToString();

                    // ڑzXg󕶎̏ꍇ
                    if (DestAddress == String.Empty)
                    {
                        throw new ArgumentException("]zXgw肳Ă܂", "DestAddress");
                    }
                }

                // ڑ|[gw肳Ăꍇ
                if (config.Contains(ConfigDestPortKey))
                {
                    try
                    {
                        // ڑ|[g擾
                        DestPort = Int32.Parse(config[ConfigDestPortKey].ToString());
                    }
                    // |[gԍɕϊłȂꍇ
                    catch (InvalidCastException e)
                    {
                        Debug.Assert(false, "ڑ|[gԍsȒlł", e.ToString());

                        // ̂܂܃ftHg|[g𗘗p
                    }

                    // |[gԍLȃ|[g͈͊Oɐݒ肳Ăꍇ
                    if (DestPort < IPEndPoint.MinPort || DestPort > IPEndPoint.MaxPort)
                    {
                        // G[𔭐
                        throw new ArgumentOutOfRangeException("DestPort", DestPort, "]|[gԍsȒlłB");
                    }
                }

                // [JAhXw肳Ăꍇ
                if (config.Contains(ConfigSourceAddressKey))
                {
                    // [JAhX擾
                    LocalAddress = config[ConfigSourceAddressKey].ToString();

                    // [JAhX󕶎̏ꍇ
                    if (LocalAddress == String.Empty)
                    {
                        // ftHg̃[JAhX𗘗p
                        LocalAddress = DefaultLocalAddress;
                    }
                }

                // [J|[gw肳Ăꍇ
                if (config.Contains(ConfigSourcePortKey))
                {
                    try
                    {
                        // [J|[gԍ擾
                        LocalPort = Int32.Parse(config[ConfigSourcePortKey].ToString());
                    }
                    // |[gԍɕϊłȂꍇ
                    catch (InvalidCastException e)
                    {
                        Debug.Assert(false, "M|[gԍsȒlł", e.ToString());

                        // ̂܂܃ftHg|[g𗘗p
                    }

                    // |[gԍLȃ|[g͈͊Oɐݒ肳Ăꍇ
                    if (LocalPort < IPEndPoint.MinPort || LocalPort > IPEndPoint.MaxPort)
                    {
                        // G[𔭐
                        throw new ArgumentOutOfRangeException("LocalPort", LocalPort, "M|[gԍsȒlłB");
                    }
                }

                // MpGR[fBOw肳Ăꍇ
                if (config.Contains(ConfigEncodingKey))
                {
                    try
                    {
                        // R[h擾
                        encode = Encoding.GetEncoding(config[ConfigEncodingKey].ToString());
                    }
                    catch (NotSupportedException e)
                    {
                        Debug.Assert(false, "sȕGR[fBOw肳Ă܂B", e.ToString());

                        Trace.WriteLine("sȕGR[fBOw肳Ă܂B", "Syslog.Collector.Writer.SimpleSyslogForwarder");
                        Trace.Indent();
                        Trace.WriteLine(e, "Syslog.Collector.Writer.SimpleSyslogForwarder");
                        Trace.Unindent();
                    }
                }

                // ő]TCYw肳Ăꍇ
                if (config.Contains(ConfigMaxForwardingSizeKey))
                {
                    try
                    {
                        // ő]TCY擾
                        MaxForwardingSize = Int32.Parse(config[ConfigMaxForwardingSizeKey].ToString());
                    }
                    catch (NotSupportedException e)
                    {
                        Debug.Assert(false, "ő]TCYsȒlłB", e.ToString());
                    }

                    // ő]TCYftHgTCY 0 ł͂Ȃꍇ
                    if (MaxForwardingSize < DefaultMaxForwardingSize && MaxForwardingSize != 0)
                    {
                        // ő]TCYftHgTCYɊg傷
                        MaxForwardingSize = DefaultMaxForwardingSize;
                    }
                }

                // [JC^[tFCX\ IPEndPoint 쐬
                LocalEP = new IPEndPoint(IPAddress.Parse(LocalAddress), LocalPort);
            }
            // JX^ݒ肪؂Ȃꍇ
            else
            {
                // G[
                throw new ArgumentException("]zXgw肳Ă܂");
            }
        }

        #region ISyslogWriter o
        /// <summary>
        /// Syslog bZ[W]邽߂̃|[gJ܂B
        /// </summary>
        public void Open()
        {
            // ł UdpClient Ăꍇ
            if (Forward != null)
            {
                // x UdpClient 
                Forward.Close();

                Forward = null;
            }

            // w肳ꂽ[JC^[tFCX UdpClient 
            Forward = new UdpClient(LocalEP);
        }

        /// <summary>
        /// sɃ^[܂B
        /// </summary>
        public void Flush()
        {
        }

        /// <summary>
        /// |[g\[X܂B̃\bh Dispose() Ăяo܂B
        /// </summary>
        public void Close()
        {
            this.Dispose();
        }

        /// <summary>
        /// Syslog bZ[W 𑼂̃VXeɓ]܂B
        /// </summary>
        /// <param name="message">̃VXeɓ] Syslog bZ[Wi[ <see cref="SyslogInfo"/>B</param>
        public void WriteSyslog(SyslogInfo message)
        {
            // UdpClient Ăꍇ
            if (Forward != null)
            {
                // w肳ꂽGR[fBO Syslog bZ[W byte zɃGR[h
                byte[] ForwardingData = encode.GetBytes(message.Message.ToString());

                // J}ŋ؂ꂽzXg̃XgɂׂđM
                foreach (string host in DestAddress.Split(','))
                {
                    // 󕶎ȊO݂̕ꍇ
                    if (host.Trim() != String.Empty)
                    {
                        try
                        {
                            // ]郁bZ[W̃TCY擾
                            int ForwardingSize = ForwardingData.Length;

                            // ő]TCYł͂Ȃɍő]TCYݒ]郁bZ[WTCY傫ꍇ
                            if (MaxForwardingSize != 0 && ForwardingSize > MaxForwardingSize)
                            {
                                // ]郁bZ[W̃TCYő]TCYɐ
                                ForwardingSize = MaxForwardingSize;
                            }

                            // w肳ꂽzXgɌăbZ[W𑗐M
                            Forward.Send(ForwardingData, ForwardingSize, host.Trim(), DestPort);
                        }
                        // MɃG[ꍇ
                        catch (SocketException e)
                        {
                            Debug.Assert(false, "Syslog pPbg]ł܂ł", e.ToString());

                            Trace.WriteLine(String.Format("{0} ֓]ł܂łB", host.Trim()), "Syslog.Collector.Writer.SimpleSyslogForwarder");
                            Trace.Indent();
                            Trace.WriteLine(e, "Syslog.Collector.Writer.SimpleSyslogForwarder");
                            Trace.Unindent();

                            // ̂܂܎̃zXg̏Ɉڍs
                        }
                    }
                    // zXg󕶎̏ꍇ
                    else
                    {
                        // ̂܂܎̃zXg̏Ɉڍs
                    }
                }
            }
        }

        /// <summary>
        /// SimpleSyslogForwarder ̃thԂ܂B
        /// </summary>
        public string Name
        {
            get
            {
                return DisplayName;
            }
        }
        #endregion

        #region IDisposable o
        /// <summary>
        /// CX^XmۂĂ郊\[X܂B
        /// </summary>
        public void Dispose()
        {
            try
            {
                // UdpClient CX^XĂꍇ
                if (Forward != null)
                {
                    // |[gă\[X
                    Forward.Close();

                    Forward = null;
                }
            }
            // \[X̉ɃG[ꍇ
            catch
            {
                // Ă̂܂ܑs
            }
        }
        #endregion

        /// <summary>
        /// JX^ݒǂݎݒi[ <see cref="Hashtable"/> Ԃ܂B
        /// </summary>
        /// <param name="setting">JX^ݒLqXMLeLXgB</param>
        /// <returns>JX^ݒi[ <see cref="Hashtable"/>BJX^ݒǂݎȂꍇ nullB</returns>
        private Hashtable GetCustomSettingNode(string setting)
        {
            XmlDocument doc = new XmlDocument();
            // JX^ݒ蕶ǂݎ
            doc.LoadXml(setting);

            // qm[h݂ꍇ
            if (doc.HasChildNodes)
            {
                // ׂĂ̎qm[h
                foreach (XmlNode node in doc.ChildNodes)
                {
                    // m[hGgŃ[J ConfigForwardKey ̃Gg̏ꍇ
                    if (node.NodeType == XmlNodeType.Element && node.LocalName == ConfigForwardKey)
                    {
                        // JX^ݒǂݍ Hashtable Ԃ
                        return (Hashtable) new SingleTagSectionHandler().Create(null, null, node);
                    }
                }
            }

            // JX^ݒ肪Ȃꍇ null ԂB
            return null;
        }
    }
}
