using System;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Xml;

namespace Syslog.Console.Controls.ConfigCreator
{
	/// <summary>
	/// AssemblyElementCreator ̊Tv̐łB
	/// </summary>
	public class AssemblyElementCreator
	{
        private string AssemblyID = String.Empty;
        private string AssemblyTypeName = String.Empty;
        private string AssemblyName = String.Empty;
        private string PublicKey = null;
        private CultureInfo CultureName = null;
        private Version AssemblyVersion = null;
        private string AssemblyPath = null;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="id"></param>
        /// <param name="type"></param>
        /// <param name="name"></param>
        /// <exception cref="ArgumentNullException"></exception>
		public AssemblyElementCreator(string id, string type, string name)
		{
            if (id == null)
            {
                throw new ArgumentNullException("id");
            }

            if (type == null)
            {
                throw new ArgumentNullException("type");
            }

            if (name == null)
            {
                throw new ArgumentNullException("name");
            }

            AssemblyID = id;
            AssemblyTypeName = type;
            AssemblyName = name;
		}

        public string ID
        {
            get
            {
                return AssemblyID;
            }
        }

        public string Type
        {
            get
            {
                return AssemblyTypeName;
            }
        }

        public string Name
        {
            get
            {
                return AssemblyName;
            }
        }

        public string PublicKeyToken
        {
            get
            {
                return PublicKey;
            }
            set
            {
                PublicKeyToken = value;
            }
        }

        public string Culture
        {
            get
            {
                if (CultureName != null)
                {
                    return CultureName.ToString();
                }
                else
                {
                    return null;
                }
            }
            set
            {
                if (value != null)
                {
                    CultureName = new CultureInfo(value);
                }
                else
                {
                    CultureName = null;
                }
            }
        }

        public string Version
        {
            get
            {
                if (AssemblyVersion != null)
                {
                    return AssemblyVersion.ToString();
                }
                else
                {
                    return null;
                }
            }
            set
            {
                if (value != null)
                {
                    AssemblyVersion = new Version(value);
                }
                else
                {
                    AssemblyVersion = null;
                }
            }
        }

        public string Path
        {
            get
            {
                return AssemblyPath;
            }
            set
            {
                if (value != null)
                {
                    System.IO.Path.GetFullPath(value);
                }

                AssemblyPath = value;
            }
        }

        public XmlNode ToNode()
        {
            return this.ToNode(String.Empty);
        }

        public XmlNode ToNode(string innerConfig)
        {
            if (innerConfig == null || innerConfig == String.Empty)
            {
                return this.ToNode((XmlNode) null);
            }
            else
            {
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(innerConfig);

                return this.ToNode(doc.DocumentElement);
            }
        }
        
        public XmlNode ToNode(XmlNode innerConfig)
        {
            const string ConfigAssembly = "assembly";
            const string ConfigAssemblyID = "id";
            const string ConfigAssemblyType = "type";
            const string ConfigAssemblyName = "name";
            const string ConfigPublicKey = "publicKeyToken";
            const string ConfigCulture = "culture";
            const string ConfigAssemblyVersion = "version";
            const string ConfigAssemblyPath = "path";

            XmlDocument doc = new XmlDocument();
            XmlElement AssemblyElement = doc.CreateElement(ConfigAssembly);

            XmlAttribute IDAttribute = doc.CreateAttribute(ConfigAssemblyID);
            IDAttribute.Value = this.ID;

            XmlAttribute TypeAttribute = doc.CreateAttribute(ConfigAssemblyType);
            TypeAttribute.Value = this.Type;

            XmlAttribute NameAttribute = doc.CreateAttribute(ConfigAssemblyName);
            NameAttribute.Value = this.Name;

            AssemblyElement.Attributes.Append(IDAttribute);
            AssemblyElement.Attributes.Append(TypeAttribute);
            AssemblyElement.Attributes.Append(NameAttribute);

            if (this.PublicKeyToken != null)
            {
                XmlAttribute PublicKeyAttribute = doc.CreateAttribute(ConfigPublicKey);
                PublicKeyAttribute.Value = this.PublicKeyToken;

                AssemblyElement.Attributes.Append(PublicKeyAttribute);
            }

            if (this.Culture != null)
            {
                XmlAttribute CultureAttribute = doc.CreateAttribute(ConfigCulture);
                CultureAttribute.Value = this.Culture;

                AssemblyElement.Attributes.Append(CultureAttribute);
            }

            if (this.Version != null)
            {
                XmlAttribute AssemblyVersionAttribute = doc.CreateAttribute(ConfigAssemblyVersion);
                AssemblyVersionAttribute.Value = this.Version;

                AssemblyElement.Attributes.Append(AssemblyVersionAttribute);
            }

            if (this.Path != null)
            {
                XmlAttribute AssemblyPathAttribute = doc.CreateAttribute(ConfigAssemblyPath);
                AssemblyPathAttribute.Value = this.Path;

                AssemblyElement.Attributes.Append(AssemblyPathAttribute);
            }

            if (innerConfig != null)
            {
                XmlNode ImportedNode = doc.ImportNode(innerConfig, true);

                AssemblyElement.AppendChild(ImportedNode);
            }

            doc.AppendChild(AssemblyElement);

            return (XmlNode) doc.DocumentElement;
        }

        public override string ToString()
        {
            return this.ToString(String.Empty);
        }

        public string ToString(string innerConfig)
        {
            return this.ToNode(innerConfig).OwnerDocument.OuterXml;
        }
	}
}
