using System;
using System.Diagnostics;
using System.ServiceProcess;
using Microsoft.Win32;

namespace Syslog.Console.Controls
{
	/// <summary>
	/// SyslogServiceProcess ̊Tv̐łB
    /// </summary>
    
	public class SyslogServiceProcess
	{
        private const string LocalMachineName = ".";
        private const string ServiceID = "mLogServer";
        private const string ServiceRegistryKey = @"SYSTEM\CurrentControlSet\Services\mLogServer";
        private const string StartupModeRegistryKey = "Start";

        private ServiceController ServiceProcess = null;

        public SyslogServiceProcess()
        {
            this.GetServiceProcess(LocalMachineName);
        }

		public SyslogServiceProcess(string machineName)
		{
            this.GetServiceProcess(machineName);
		}

        public ServiceControllerStatus Status
        {
            get
            {
                return ServiceProcess.Status;
            }
        }

        public ServiceStartupMode StartupMode
        {
            get
            {
                using (RegistryKey rk = Registry.LocalMachine.OpenSubKey(ServiceRegistryKey))
                {
                    return (ServiceStartupMode) rk.GetValue(StartupModeRegistryKey);
                }
            }
            set
            {
                using (RegistryKey rk = Registry.LocalMachine.OpenSubKey(ServiceRegistryKey, true))
                {
                    rk.SetValue(StartupModeRegistryKey, (int) value);

                    rk.Flush();
                }
            }
        }

        public void Start()
        {
            ServiceProcess.Start();

            this.Refresh();
        }

        public void Start(string[] args)
        {
            ServiceProcess.Start(args);

            this.Refresh();
        }

        public void Stop()
        {
            ServiceProcess.Stop();

            this.Refresh();
        }

        public void Pause()
        {
            ServiceProcess.Pause();

            this.Refresh();
        }

        public void Continue()
        {
            ServiceProcess.Continue();

            this.Refresh();
        }

        public void Refresh()
        {
            ServiceProcess.Refresh();
        }

        public bool CanPauseAndContinue
        {
            get
            {
                return ServiceProcess.CanPauseAndContinue;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="machineName"></param>
        /// <exception cref="InvalidOperationException">Managed Syslog Server T[rXȂƂɃX[܂B</exception>
        private void GetServiceProcess(string machineName)
        {
            ServiceProcess = new ServiceController(ServiceID, machineName);

            if (ServiceProcess.ServiceName == ServiceID)
            {
            }
        }
	}
}
