using System;
using System.Diagnostics;
using System.Threading;
using Syslog.Listener;
using Syslog.Relay;

namespace Syslog.Service
{
	/// <summary>
	/// Syslog Controller 𐧌䂵 Syslog bZ[WK؂ȏꏊɕۑ܂
	/// </summary>
	public class ServiceComponent : IDisposable
	{
        private Thread ServiceThread = null;
        private TimeSpan BlockingTime = new TimeSpan(0, 0, 10);
        private Exception exception = null;

        /// <summary>
        /// ftHgRXgN^B
        /// </summary>
		public ServiceComponent()
		{
		}

        /// <summary>
        /// Managed Syslog Server C[vłB
        /// </summary>
        protected internal void ProcessingLoop()
        {
            Trace.WriteLine("Initializing RelayController...", "Syslog.Service.ServiceComponent");

            try
            {
                // [Rg[CX^X
                using (RelayController relay = new RelayController())
                {
                    Trace.WriteLine("Initializing ListenerController...", "Syslog.Service.ServiceComponent");

                    // XiRg[CX^X
                    using (ListenerController listen = new ListenerController())
                    {
                        // MCxgƓ邽߂̃VOiOIuWFNg擾
                        ServiceRunState.Signal = listen.WaitSignal;

                        // T[rX~Ԃł͂Ȃꍇ͏JԂ
                        while (ServiceRunState.Status != ServiceStatus.OnStop)
                        {
                            // T[rX̓Xe[^X`FbNăpPbg̏Jn邩肷
                            switch (ServiceRunState.Status)
                            {
                                    // T[rXꎞ~Ăꍇ
                                case ServiceStatus.OnPause:
                                {
                                    // pPbg̑҂󂯂𒆒f
                                    listen.Pause();

                                    Trace.WriteLine("Paused.", "Syslog.Service.ServiceComponent");

                                    // T[rXĊJ̃VOi܂ő҂
                                    ServiceRunState.Signal.WaitOne();

                                    // [vx]
                                    continue;
                                }
                                    // T[rXĊJĂꍇ
                                case ServiceStatus.OnContinue:
                                {
                                    // pPbg̑҂󂯂Jn
                                    listen.Continue();

                                    // T[rXsԂɂ
                                    ServiceRunState.Status = ServiceStatus.OnStart;

                                    Trace.WriteLine("Continued.", "Syslog.Service.ServiceComponent");

                                    // [vx]
                                    continue;
                                }
                            } // T[rXsԂ̏ꍇ̓pPbg̑҂󂯏𑱍s

                            // pPbgw肳ꂽԂubLOđ҂󂯂
                            SyslogPacket packet = listen.GetPacket(BlockingTime);

                            if (packet != null)
                            {
                                // pPbg[eBO
                                relay.Relay(packet);
                            }

                            // ̃Xi Xbhɏ
                            Thread.Sleep(0);
                        } // T[rX~ԂɂȂ珈[vI

                        // Xi Rg[̃pPbg̑҂󂯂𒆎~
                        listen.Close();

                        // L[ɂ܂ĂSyslogbZ[Wׂď
                        while (true)
                        {
                            // L[SyslogbZ[Wo
                            SyslogPacket packet = listen.GetPacket(0);

                            // SyslogbZ[WcĂȂꍇ
                            if (packet == null)
                            {
                                // L[̏Iă[v𔲂
                                break;
                            }

                            // L[ɎcĂSyslogbZ[W[eBO
                            relay.Relay(packet);
                        }
                    } // Xi Rg[ Dispose() mɌĂяo
                } // [ Rg[ Dispose() mɌĂяo
            }
            // ɃG[ꍇ
            catch (Exception e)
            {
                Trace.WriteLine("Main Processing Loop Error.", "Syslog.Service.ServiceComponent");

                // ̃CX^XbN
                lock (this)
                {
                    // Oۑ
                    exception = e;
                }
            }
            
            Trace.WriteLine("Shutdowned.", "Syslog.Service.ServiceComponent");
        }

        /// <summary>
        /// C[vXbh쐬ăT[rXJnB
        /// </summary>
        public void Start()
        {
            Trace.WriteLine("Starting...", "Syslog.Service.ServiceComponent");

            // C[vVXbhŎs
            ServiceThread = new Thread(new ThreadStart(ProcessingLoop));
            // tHAOEhœ삳
            ServiceThread.IsBackground = false;
            // Xbh̖Oݒ肷
            ServiceThread.Name = "Syslog Server Main Service Thread";
            // Xbh̗Dxw肷
            ServiceThread.Priority = ThreadPriority.Normal;

            // T[rX̃Xe[^XXV
            ServiceRunState.Status = ServiceStatus.OnStart;

            try
            {
                // C[vVXbhŎsn߂
                ServiceThread.Start();

                Trace.WriteLine("Starting Main Processing Loop...", "Syslog.Service.ServiceComponent");
            }
            catch (Exception e)
            {
                Debug.Assert(false, "CXbh̐Ɏs܂",e.ToString());

                Trace.WriteLine("CXbh̐Ɏs܂!", "Syslog.Service.ServiceComponent");
                Trace.Indent();
                Trace.WriteLine(e, "Syslog.Service.ServiceComponent");
                Trace.Unindent();

                throw e;
            }

            // CXbhSɋN̂҂
            this.WaitStateChange();

            Trace.WriteLine("Main Processing Loop Started.", "Syslog.Service.ServiceComponent");
        }

        /// <summary>
        /// T[rX~B
        /// </summary>
        public void Stop()
        {
            this.Dispose();
        }

        /// <summary>
        /// T[rX̓ĊJB
        /// </summary>
        public void Continue()
        {
            Trace.WriteLine("Continuing...", "Syslog.Service.ServiceComponent");

            // T[rX̃Xe[^XXV
            ServiceRunState.Status = ServiceStatus.OnContinue;

            // T[rXĊJ̃VOi𑗂
            ServiceRunState.Signal.Set();

            this.WaitStateChange();
        }

        /// <summary>
        /// T[rX̓ꎞ~B
        /// </summary>
        public void Pause()
        {
            Trace.WriteLine("Pausing...", "Syslog.Service.ServiceComponent");

            // T[rX̃Xe[^XXV
            ServiceRunState.Status = ServiceStatus.OnPause;

            // ҂󂯂~ăubLOɓ邽߂̃VOi𑗂
            ServiceRunState.Signal.Set();

            this.WaitStateChange();
        }

        #region IDisposable o
        /// <summary>
        /// CX^XێĂ郊\[X܂B
        /// </summary>
        public void Dispose()
        {
            Trace.WriteLine("Shutdowning...", "Syslog.Service.ServiceComponent");

            // T[rX̃Xe[^X~Ԃɐݒ肵܂
            ServiceRunState.Status = ServiceStatus.OnStop;

            // CXbhNĂȂꍇ
            if (ServiceThread == null)
            {
                Debug.Assert(false, "CXbh݂܂");

                // \[X̉I
                return;
            }

            // C[vXbhsł͂Ȃꍇ
            if (!ServiceThread.IsAlive)
            {
                // XbhI
                ServiceThread.Abort();
                // Xbh̏Iɐ䌠
                Thread.Sleep(0);
                // Xbh̏I10bԂubLOđ҂
                ServiceThread.Join(new TimeSpan(0, 0, 10));

                // \[X̉I
                return;
            }

            // CXbhɃVOi𑗂ăpPbg̑҂󂯂؂グ
            ServiceRunState.Signal.Set();

            // Xbhł̏15bԂubLOđ҂
            if (!ServiceThread.Join(new TimeSpan(0, 0, 15)))
            {
                // XbhI
                ServiceThread.Abort();
                // Xbh̏Iɐ䌠
                Thread.Sleep(0);
                // Xbh̏I5bԂubLOđ҂
                ServiceThread.Join(new TimeSpan(0, 0, 5));

                Trace.WriteLine("ML[ɎcĂ郁bZ[W؂܂łB", "Syslog.Service.ServiceComponent");
            }
        }
        #endregion

        private bool WaitStateChange()
        {
            // C[vI܂őҋ@
            for (int i = 0; i < 20; i++)
            {
                // I܂1bԑ҂
                Thread.Sleep(1000);

                // XbhIĂ܂Ăꍇ
                if (!ServiceThread.IsAlive)
                {
                    Debug.Assert(false, "CXbhُI܂", exception.ToString());

                    Trace.WriteLine("CXbhُI܂!", "Syslog.Service.ServiceComponent");
                    Trace.Indent();
                    Trace.WriteLine(exception, "Syslog.Service.ServiceComponent");
                    Trace.Unindent();

                    // ۑOoăX[
                    throw exception;
                }

                // CXbhM҂Ɉڍsꍇ
                if ((ServiceThread.ThreadState & System.Threading.ThreadState.WaitSleepJoin) != 0)
                {
                    // Nҋ@[v𔲂
                    return true;
                }
            }

            // IĂȂ\
            return false;
        }
    }
}
