using System;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;
using System.Diagnostics;
using System.Xml;

namespace Syslog.Config.EventLog.SectionHandler
{
    /// <summary>
    /// App.config \ZNṼGgǂݎ<see cref="EventLogPriorityEntry"/> ̔z <see cref="ArrayList"/> ƂĕԂ܂B
    /// </summary>
    /// <remarks>
    /// <example>
    /// <para>\ZNV̋Lq</para>
    /// <code>
    /// &lt;eventlog&gt;
    ///   &lt;priority facility="User" severity="Notice"&gt;
    ///     &lt;regex source="EntryType"&gt;Info.*&lt;/regex&gt;
    ///   &lt;/priority&gt;
    /// &lt;eventlog&gt;
    /// </code>
    /// </example>
    /// </remarks>
	public class EventLogPriorityListSectionHandler : System.Configuration.IConfigurationSectionHandler
	{
        private const string priorityKey = "priority";
        private const string facilityKey = "facility";
        private const string severityKey = "severity";

        /// <summary>
        /// ftHgRXgN^B
        /// </summary>
		public EventLogPriorityListSectionHandler()
		{
        }

        /// <summary>
        /// \ZNVǂݎ<see cref="EventLogPriorityEntry"/> ̔z <see cref="ArrayList"/> ƂĕԂ܂B
        /// </summary>
        /// <param name="node">\ZNV <see cref="XmlNode"/> ̕\B</param>
        /// <returns>\ZNVϊ <see cref="EventLogPriorityEntry"/> ̔zB</returns>
        public static object Parse(string node)
        {
            // \ZNVɃf[^ꍇ
            if (node != null || node != String.Empty)
            {
                XmlDocument doc = new XmlDocument();
                XmlDocumentFragment frag = doc.CreateDocumentFragment();
                // m[hhLgtOgƂēǂݍ
                frag.InnerXml = node;

                // CX^X쐬
                EventLogPriorityListSectionHandler handler = new EventLogPriorityListSectionHandler();
                // \ZNVIPEndPoint̔zԂ
                return handler.Create(null, null, frag);
            }
                // \ZNṼf[^̏ꍇ
            else
            {
                // ̔z쐬ĕԂ
                return new EventLogPriorityListSectionHandler().Create(null, null, null);
            }
        }

        #region IConfigurationSectionHandler o
        /// <summary>
        /// \ZNVǂݎ <see cref="EventLogPriorityEntry"/> ̔z <see cref="ArrayList"/> Ƃč쐬܂B
        /// </summary>
        /// <param name="parent">Ήe̍\ZNVB</param>
        /// <param name="configContext">\p[^B null QƂłB</param>
        /// <param name="section">\ZNVi[Ă <see cref="XmlNode"/>B</param>
        /// <returns>\ZNV̎qGgǂݎ <see cref="EventLogPriorityEntry"/> ̔zB</returns>
        public object Create(object parent, object configContext, XmlNode section)
        {
            ArrayList list = new ArrayList();

            // \ZNṼf[^Ȃꍇ
            if (section == null)
            {
                return list;
            }

            // qm[hꍇ
            if (section.HasChildNodes)
            {
                // qm[hׂď
                foreach (XmlNode node in section.ChildNodes)
                {
                    // qm[hGgł͂Ȃꍇ
                    if (node.NodeType != XmlNodeType.Element)
                    {
                        // ̃m[hփXLbv
                        continue;
                    }

                    // m[h̃[J priorityKey ł͂Ȃꍇ
                    if (node.LocalName != priorityKey)
                    {
                        // ̃m[hփXLbv
                        continue;
                    }

                    // Facility Agr[g擾
                    XmlAttribute FacilityAttribute = node.Attributes[facilityKey];
                    // Severity Agr[g擾
                    XmlAttribute SeverityAttribute = node.Attributes[severityKey];

                    // EventLogPriorityEntry 쐬
                    EventLogPriorityEntry entry = new EventLogPriorityEntry();

                    // Facility lw肳Ăꍇ
                    if (FacilityAttribute != null)
                    {
                        // Facility ݒ肷
                        entry.Facility = Priority.ParseFacility(FacilityAttribute.Value).ToString();
                    }

                    // Severity lw肳Ăꍇ
                    if (SeverityAttribute != null)
                    {
                        // Severity ݒ肷
                        entry.Severity = Priority.ParseSeverity(SeverityAttribute.Value).ToString();
                    }

                    // qm[h炳ɐK\}b`Gg̔z쐬
                    entry.Regexes = (ArrayList) new EventLogRegexListSectionHandler().Create(section, null, node);

                    // EventLogPriorityEntry ǉ
                    list.Add(entry);
                }
            }

            // EventLogPriorityEntry ̔zԂ
            return list;
        }
        #endregion
    }
}
