using System;
using System.Diagnostics;
using System.Text.RegularExpressions;

namespace Syslog.Config.EventLog
{
	/// <summary>
	/// app.config \t@C琳K\}b`Ggǂݍ݂܂B
    /// </summary>
    /// <remarks>
    /// <example>
    /// <para>\ZNV̋Lq</para>
    /// <code>
    /// &lt;eventlog&gt;
    ///   &lt;priority facility="User" severity="Notice"&gt;
    ///     &lt;regex source="EntryType"&gt;Info.*&lt;/regex&gt;
    ///   &lt;/priority&gt;
    /// &lt;eventlog&gt;
    /// </code>
    /// </example>
    /// </remarks>
	public class EventLogRegexEntry
	{
        // }b`̂߂̓t\tH[}bg
        private const string DateTimeFormat = "s";
        private const RegexOptions RegexOption = RegexOptions.ExplicitCapture | RegexOptions.IgnoreCase | RegexOptions.IgnorePatternWhitespace | RegexOptions.Singleline;

        private EventLogEventSourceType eventSource = EventLogEventSourceType.EntryType;
        private Regex eventRegex = null;

        /// <summary>
        /// ftHgRXgN^
        /// </summary>
		public EventLogRegexEntry()
		{
        }

        /// <summary>
        /// K\}b`CxgGg̍ڂ\܂B
        /// </summary>
        public string EventSource
        {
            get
            {
                return eventSource.ToString();
            }
            set
            {
                // \啶ʂɗ񋓌^ɕϊ܂
                eventSource = (EventLogEventSourceType) Enum.Parse(typeof(EventLogEventSourceType), value, true);
            }
        }

        /// <summary>
        /// K\}b`ɗp鐳K\p^[B
        /// </summary>
        public string RegexPattern
        {
            get
            {
                return eventRegex.ToString();
            }
            set
            {
                if (value != null)
                {
                    eventRegex = new Regex(value, RegexOption);
                }
                else
                {
                    eventRegex = null;
                }
            }
        }

        /// <summary>
        /// }b`GgɃCxgOGg}b`邩ׂ܂B
        /// </summary>
        /// <param name="entry">}b`邩ׂ <see cref="EventLogEntry"/>B</param>
        /// <returns>}b`ꍇ trueB}b`Ȃꍇ falseB</returns>
        public bool IsMatch(EventLogEntry entry)
        {
            string SourceValue = entry.Source;

            #region CxgÕf[^擾
            switch (eventSource)
            {
                // CxgID
                case EventLogEventSourceType.EventID:
                {
                    SourceValue = entry.EventID.ToString();
                    break;
                }
                // Cxg̎
                case EventLogEventSourceType.EntryType:
                {
                    SourceValue = entry.EntryType.ToString();
                    break;
                }
                // Cxg̃Cxg\[X
                case EventLogEventSourceType.Source:
                {
                    SourceValue = entry.Source;
                    break;
                }
                // Cxg𔭐[U
                case EventLogEventSourceType.UserName:
                {
                    SourceValue = entry.UserName;
                    break;
                }
                // Cxg𔭐Rs[^
                case EventLogEventSourceType.Hostname:
                {
                    SourceValue = entry.MachineName;
                    break;
                }
                // Cxg
                case EventLogEventSourceType.DateTime:
                {
                    SourceValue = entry.TimeGenerated.ToString(DateTimeFormat);
                    break;
                }
                // Cxg̃JeSԍ
                case EventLogEventSourceType.CategoryID:
                {
                    SourceValue = entry.CategoryNumber.ToString();
                    break;
                }
                // Cxg̃JeS
                case EventLogEventSourceType.Category:
                {
                    try
                    {
                        // JeS擾
                        SourceValue = entry.Category;
                    }
                    // JeS擾łȂꍇ
                    catch
                    {
                        // JeS󕶎ɐݒ
                        SourceValue = String.Empty;
                    }

                    break;
                }
                // Cxg̃bZ[We
                case EventLogEventSourceType.Message:
                {
                    try
                    {
                        // bZ[We擾
                        SourceValue = entry.Message;
                    }
                    // bZ[We擾łȂꍇ
                    catch
                    {
                        // bZ[We󕶎ɐݒ
                        SourceValue = String.Empty;
                    }

                    break;
                }
            }
            #endregion

            // }b`ɗp鐳K\ null ̏ꍇ
            if (eventRegex == null)
            {
                // `FbNs킸Ƀ}b`Ƃɂ
                return true;
            }

            // }b`镶 null ̏ꍇ
            if (SourceValue == null)
            {
                // null  󕶎ɒu
                SourceValue = String.Empty;
            }

            // K\}b`sĂ̌ʂԂ
            return eventRegex.IsMatch(SourceValue);
        }
	}
}
