using System;
using System.Globalization;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using Syslog.Config;

namespace Syslog
{
	/// <summary>
	/// Syslog pPbg\Ċi[܂B
	/// </summary>
	[Serializable]
	public class SyslogMessage : ICloneable
    {
        #region Dx K\p^[
        //  < Ŏn܂擪 0 ̏ꍇ͌㑱ɐ 3 ̐lōŌオ > ̏ꍇɃ}b`
        const string PriorityRegexPattern = @"
                \A<                                     # ̐擪 < 
                    (?<PRI>                             # Dx PRI ɃLv`
                        (?!0[0-9])                      # 擪 0 tƂ͂Ȃ
                        ([0-9]{1,2}|1[0-8][0-9]|19[01]) # 3̐ : ő 191 ܂
                    )
                >                                       # Ō > ";
        #endregion

        #region ^CX^v K\p^[
        const string TimeStampRegexPattern = @"
                (?=[JFMASOND])                                          # 擪̖Öꕔ̏ꍇ
                (?<TimeStamp>                                           # ^CX^v TimeStamp ɃLv`
                    (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)   # ̖Öv
                    \                                                   # Xy[X1
                    (\ [1-9]|[12][0-9]|3[01])                           # 131܂ : ꌅ̏ꍇ͐擪ɃXy[X1t
                    \                                                   # Xy[X1
                    ([01][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]            #  : 00:00:00  23:59:59 ܂
                )";
        #endregion

        #region zXg K\p^[
        const string HostAddressPattern = @"
                (?<Host>                                                        # zXg Host ɃLv`
                    (?=[a-zA-Z0-9:])                                            # 擪zXgɎg镶//IPv6ȗL:̏ꍇ
                    (                                                           # zXg/IPv4AhX/IPv6AhX̂ꂩ}b`
                        ([a-zA-z]([-a-zA-Z0-9]{0,61}[a-zA-Z0-9])?)              # zXg: p->pƃnCt->p/63ȉ
                    |
                        (22[0-3]|2[01][0-9]|10[0-9]|1?[0-9][0-9]|[1-9])         # IPv4AhX: NXC(223.*)ȉƃ}b`
                        (                                                       #               擪0̏ꍇ̓}b`Ȃ
                            \.                                                  #               .ŋ؂
                            (25[0-5]|2[0-4][0-9]|10[0-9]|1?[1-9][0-9]|[1-9])    #               1255̊Ԃ̐l
                        ){3}                                                    #               3JԂ
                    |
                        ([:.a-fA-F0-9]{2,39})                                   # IPv6AhX: ȈՃ}b`/ׂĂɃ}b`
                    )
                )";
        #endregion

        #region bZ[W{ K\p^[
        const string MessagePattern = @"
                (?>
                    (?<MSG>             # bZ[W MSG ɃLv`
                        \P{Cn}*         # 蓖ăR[hȊÔׂĂƃ}b`
                    )
                )\z                     # ̖܂Ń}b`";
        #endregion

        #region SyslogpPbg K\p^[
        const string SyslogPattern = @"
                (?>
                    (                                       # LPRIwb_Ȃꍇ" +
                        PriorityRegexPattern + @"
                        (?>
                            (                               # Lȃ^CX^vȂꍇ" +
                                TimeStampRegexPattern + @"
                                \                           # ^CX^vƃzXg̊ԂɃXy[X1
                                (                           # LȃzXgȂꍇ" +
                                    HostAddressPattern + @"
                                    \                       # zXĝƂɃXy[X1
                                )?
                            )?
                        )
                    )?
                )" + MessagePattern + @"                    # ׂ̑ĂbZ[WƂă}b`";
        #endregion

        // ftHg RegexOptions ̒lw肷
        private const RegexOptions RegexOptionGroup =
            RegexOptions.Compiled | RegexOptions.CultureInvariant | RegexOptions.ExplicitCapture | RegexOptions.IgnorePatternWhitespace | RegexOptions.Singleline;

        private static Regex SyslogRegex = new Regex(SyslogPattern, RegexOptionGroup);

        private Facility facility = Syslog.Facility.User;
        private Severity severity = Syslog.Severity.Notice;
        private DateTime timestamp = DateTime.Parse(DateTime.Now.ToString("s"));
        private string hostname = Dns.GetHostName();
        private string tag = String.Empty;
        private string content = String.Empty;

        /// <summary>
        /// ftHgRXgN^B
        /// </summary>
		public SyslogMessage()
		{
		}

        /// <summary>
        /// RXgN^B <see cref="SyslogPacket"/> ǂݎCX^X쐬܂B
        /// </summary>
        /// <param name="packet">CX^Xɓǂݍ <see cref="SyslogPacket"/>B</param>
        /// <exception cref="System.ArgumentNullException"><c>packet</c>  null QƂłB</exception>
        public SyslogMessage(SyslogPacket packet)
        {
            // pPbg񂪂Ȃꍇ
            if (packet == null)
            {
                throw new ArgumentNullException("packet");
            }

            string packetData = null;

            // pPbg̃f[^Ȃꍇ
            if (packet.PacketData != null)
            {
                packetData = packet.GetPacketString();
            }
            else
            {
                packetData = String.Empty;
            }

            // pPbg̑M񂪂Ȃꍇ
            if (packet.RemoteEndPoint != null)
            {
                this.Init(packetData, packet.RemoteEndPoint.Address.ToString());
            }
            else
            {
                this.Init(packetData, null);
            }
        }

        /// <summary>
        /// RXgN^B<see cref="SyslogPacket"/>  <c>remoteHost</c>ɃCX^X쐬܂B
        /// </summary>
        /// <param name="packetData">CX^Xɓǂݍ <see cref="SyslogPacket"/>B</param>
        /// <param name="remoteHost"></param>
        /// <exception cref="System.ArgumentNullException"><c>pakcet</c>  null QƂłB</exception>
        internal SyslogMessage(string packetData, string remoteHost)
        {
            // pPbg񂪂Ȃꍇ
            if (packetData == null)
            {
                throw new ArgumentNullException("packet");
            }

            this.Init(packetData, remoteHost);
        }

        /// <summary>
        /// CX^X̏܂B
        /// </summary>
        /// <param name="packetData">CX^X̏ɗp <see cref="SyslogPacket"/> ̃pPbgf[^B</param>
        /// <param name="remoteHost">pPbg̑MzXg̃AhXB</param>
        /// <exception cref="System.ArgumentNullException"><c>packetData</c>  null QƂłB</exception>
        private void Init(string packetData, string remoteHost)
        {
            // <PRI>TIMESTAMP HOSTNAME TAG:CONTENT
            // <000>Nov 12 12:00:00 gateway TAG:CONTENT

            if (packetData == null)
                throw new ArgumentNullException("packetData");

            // pPbgtH[}bg`FbN
            Match SyslogMatch = SyslogRegex.Match(packetData);

            Group PriorityGroup = SyslogMatch.Groups["PRI"];
            Group TimeStampGroup = SyslogMatch.Groups["TimeStamp"];
            Group HostAddressGroup = SyslogMatch.Groups["Host"];
            Group MsgGroup = SyslogMatch.Groups["MSG"];

            // pPbgtH[}bg̏ꍇ
            if (PriorityGroup.Success && TimeStampGroup.Success && HostAddressGroup.Success)
            {
                this.Priority = Int32.Parse(PriorityGroup.Value);
                this.SetTimeStamp(TimeStampGroup.Value);
                this.Hostname = HostAddressGroup.Value;
                // MSGݒ
                this.SetMSG(MsgGroup.Value);

            } // PRITimeStamp͗LHostnameȏꍇ
            else if (PriorityGroup.Success && TimeStampGroup.Success && !HostAddressGroup.Success)
            {
                this.Priority = Int32.Parse(PriorityGroup.Value);
                this.SetTimeStamp(TimeStampGroup.Value);
                // zXg
                this.Hostname = this.LookupHostname(remoteHost);
                // c̃f[^CONTENTɐݒ
                this.CONTENT = MsgGroup.Value;

            } // PRI͗LTimeStamp
            else if (PriorityGroup.Success && !TimeStampGroup.Success && !HostAddressGroup.Success)
            {
                this.Priority = Int32.Parse(PriorityGroup.Value);
                // ݂̎ݒ
                this.TimeStamp = DateTime.Parse(DateTime.Now.ToString("s"));
                // zXg
                this.Hostname = this.LookupHostname(remoteHost);
                // c̃f[^CONTENTɐݒ
                this.CONTENT = MsgGroup.Value;

            } // LPRIȂꍇ
            else
            {
                // ݂̎ݒ
                this.TimeStamp = DateTime.Parse(DateTime.Now.ToString("s"));
                // zXg
                this.Hostname = this.LookupHostname(remoteHost);
                // f[^ׂĂCONTENTɐݒ
                this.CONTENT = MsgGroup.Value;
            }
        }

        /// <summary>
        /// <see cref="SyslogPacket"/> ̃pPbgf[^ǂݎ <see cref="SyslogMessage"/> 쐬܂B
        /// </summary>
        /// <param name="packetData">pPbgf[^̕B</param>
        /// <returns><see cref="SyslogPacket"/> ǂݎ菉ꂽ <see cref="SyslogMessage"/> ̃CX^XB</returns>
        public static SyslogMessage Parse(string packetData)
        {
            return SyslogMessage.Parse(packetData, null);
        }

        /// <summary>
        /// <see cref="SyslogPacket"/> ̃pPbgf[^ǂݎ <see cref="SyslogMessage"/> 쐬܂B
        /// </summary>
        /// <param name="packetData">MpPbg̃oCgzB</param>
        /// <param name="encoding">pPbgf[^̕GR[fBO\ <see cref="Encoding"/>B</param>
        /// <returns><see cref="SyslogPacket"/> ǂݎ菉ꂽ <see cref="SyslogMessage"/> ̃CX^XB</returns>
        public static SyslogMessage Parse(byte[] packetData, Encoding encoding)
        {
            return SyslogMessage.Parse(packetData, Dns.GetHostName(), encoding);
        }

        /// <summary>
        /// <see cref="SyslogPacket"/> ̃pPbgf[^ǂݎ <see cref="SyslogMessage"/> 쐬܂B
        /// </summary>
        /// <param name="packetData">pPbgf[^̕B</param>
        /// <param name="remoteHost">pPbg̑MB</param>
        /// <returns><see cref="SyslogPacket"/> ǂݎ菉ꂽ <see cref="SyslogMessage"/> ̃CX^XB</returns>
        public static SyslogMessage Parse(string packetData, string remoteHost)
        {
            return new SyslogMessage(packetData, remoteHost);
        }

        /// <summary>
        /// <see cref="SyslogPacket"/> ̃pPbgf[^ǂݎ <see cref="SyslogMessage"/> 쐬܂B
        /// </summary>
        /// <param name="packetData">pPbgf[^̕B</param>
        /// <param name="remoteHost">pPbg̑MB</param>
        /// <param name="encoding">pPbgf[^̕GR[fBO\ <see cref="Encoding"/>B</param>
        /// <returns><see cref="SyslogPacket"/> ǂݎ菉ꂽ <see cref="SyslogMessage"/> ̃CX^XB</returns>
        public static SyslogMessage Parse(byte[] packetData, string remoteHost, Encoding encoding)
        {
            if (encoding != null)
                return SyslogMessage.Parse(encoding.GetString(packetData), remoteHost);
            else
                return SyslogMessage.Parse(Encoding.GetEncoding("euc-jp").GetString(packetData), remoteHost);
        }

        /// <summary>
        /// PRIp[gԂ܂B
        /// </summary>
        /// <remarks>
        /// Facility = (Facility * 8 + Severity)
        /// 擪 &lt; łȂꍇ͗LPRI܂܂ĂȂB
        /// 3, 4, 5 Ԗڂ̂ &gt; łȂꍇ͗LPRI܂܂ĂȂB
        /// Prioritysȏꍇ͗LPRI܂܂ĂȂB
        /// </remarks>
        public string PRI
        {
            get
            {
                // <[PRI]>
                return String.Format("<{0}>", this.Priority);
            }
        }

        /// <summary>
        /// HEADERp[gԂ܂B
        /// </summary>
        /// <remarks>
        /// HEADER = Timestamp Hostname
        /// </remarks>
        public string HEADER
        {
            get
            {
                // [TIMESTAMP][ ][HOSTNAME]
                return String.Format("{0} {1}", this.GetTimeStamp(), this.Hostname);
            }
        }

        /// <summary>
        /// MSGp[gԂ܂B
        /// </summary>
        /// <remarks>
        /// MSG = TAG CONTENT
        /// </remarks>
        public string MSG
        {
            get
            {
                // [TAG][CONTENT]
                return this.TAG + this.CONTENT;
            }
        }

        /// <summary>
        /// DxԂ܂B
        /// </summary>
        /// <remarks>
        /// Priority = (Facility * 8) + Severity
        /// Priority = (Facility &lt;&lt; 3) + Severity
        /// </remarks>
        /// <value>1310iB</value>
        public int Priority
        {
            get
            {
                // vCIeBvZ
                return (this.Facility * 8) + this.Severity;
            }
            set
            {
                // vCIeB 3rbgEɃVtg t@VeB vZ
                this.Facility = value >> 3;

                // vCIeB t@VeB 8{ dx vZ
                this.Severity = value - (this.Facility * 8);
            }
        }

        /// <summary>
        /// Facility / @\ Ԃ܂B
        /// </summary>
        /// <remarks>
        /// ftHg user-level (1) B
        /// Facility = Priority &gt;&gt; 3
        /// </remarks>
        /// <value>Facility 0  23 ̊ԂłB</value>
        public int Facility
        {
            get
            {
                // ݒ肳Ă t@VeB 023܂ł͈̔͂̏ꍇ
                if (0 <= (int) this.facility && (int) this.facility <= 23)
                {
                    // t@VeB Ԃ
                    return (int) this.facility;
                }
                else
                {
                    // ݒ肳Ă t@VeB 023܂ł͈̔͂ł͂Ȃꍇ
                    // ftHg Facility.User Ԃ
                    return (int) Syslog.Facility.User;
                }
            }
            set
            {
                // t@VeB ݒ
                this.facility = (Syslog.Facility) value;
            }
        }

        /// <summary>
        /// Severity / dx Ԃ܂B
        /// </summary>
        /// <remarks>
        /// ftHg Notice (5) B
        /// Severity = Priority - ((Priority &gt;&gt; 3) * 8)
        /// </remarks>
        /// <value>Severity 0  7 ̊ԂłB</value>
        public int Severity
        {
            get
            {
                if (0 <= (int) this.severity && (int) this.severity <= 7)
                {
                    return (int) this.severity;
                }
                else
                {
                    return (int) Syslog.Severity.Notice;
                }
            }
            set
            {
                this.severity = (Syslog.Severity) value;
            }
        }

        /// <summary>
        /// ^CX^vԂ܂B
        /// </summary>
        /// <value>MMM [ ]d HH:mm:ss</value>
        public DateTime TimeStamp
        {
            get
            {
                return DateTime.Now;
            }
            set
            {
                this.timestamp = value;
            }
        }

        /// <summary>
        /// zXgԂ܂B
        /// </summary>
        /// <value>IPv4, IPv6, zXg</value>
        public string Hostname
        {
            get
            {
                return this.hostname;
            }
            set
            {
                if (value == null)
                    throw new ArgumentNullException("Hostname");

                string Host = value;

                // 󔒕ݒ肳ꂽꍇ̓[JzXgݒ肷
                if (Host == String.Empty)
                    Host = this.LookupHostname(value);

                // zXgɎg镶`FbN
                Regex HostCheck = new Regex(HostAddressPattern, RegexOptionGroup);

                // zXg̖ɓKĂꍇ
                if (HostCheck.IsMatch(Host))
                {
                    this.hostname = Host;
                }
                else
                {
                    throw new ArgumentException("zXgƂėpł܂", "Hostname");
                }
            }
        }

        /// <summary>
        /// TAGԂ܂B
        /// </summary>
        /// <remarks>
        /// vO, vZXB
        /// pȊO̕܂łTAGBȍ~CONTENTB
        /// </remarks>
        /// <value>
        /// 32ȓBpB
        /// </value>
        /// <exception cref="System.ArgumentNullException">ݒ肳ꂽl null Q łB</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">ݒ肳ꂽlɉp̕܂܂Ă܂B</exception>
        public string TAG
        {
            get
            {
                return this.tag;
            }
            set
            {
                if (value == null)
                    throw new ArgumentNullException("TAG");

                // p 32܂
                Regex TAGRegex = new Regex(@"^(?<TAG>[0-9a-zA-Z]{0,32})$", RegexOptionGroup);
                Match TAGMatch = TAGRegex.Match(value);

                if (TAGMatch.Success)
                {
                    this.tag = TAGMatch.Groups["TAG"].Value;
                }
                else
                {
                    throw new ArgumentOutOfRangeException("TAG", value, "No Match.");
                }
            }
        }

        /// <summary>
        /// CONTENTԂ܂B
        /// </summary>
        /// <remarks>
        /// bZ[W̏ڍׁB
        /// '[', ':', ' ' CONTENT̎n܂ł邱ƂB
        /// </remarks>
        /// <exception cref="System.ArgumentNullException">ݒ肳ꂽl null QƂłB</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">ݒ肳ꂽlɈ\Ɋ܂܂Ȃ܂܂Ă܂B</exception>
        public string CONTENT
        {
            get
            {
                return this.content;
            }
            set
            {
                if (value == null)
                    throw new ArgumentNullException("CONTENT");

                Regex ContentRegex = new Regex(@"^(?<Content>\P{Cn}*)$", RegexOptionGroup);
                Match ContentMatch = ContentRegex.Match(value);

                if (ContentMatch.Success)
                {
                    this.content = ContentMatch.Groups["Content"].Value;
                }
                else
                {
                    throw new ArgumentOutOfRangeException("Content", value, "No Match.");
                }
            }
        }

        /// <summary>
        /// Syslog tH[}bg`ł ^CX^vlԂ܂B
        /// </summary>
        /// <returns>MMM dd HH:mm:ss `̓B</returns>
        public string GetTimeStamp()
        {
            if (this.TimeStamp.Day < 10)
            {
                return this.TimeStamp.ToString("MMM  d HH:mm:ss", DateTimeFormatInfo.InvariantInfo);
            }
            else
            {
                return this.TimeStamp.ToString("MMM dd HH:mm:ss", DateTimeFormatInfo.InvariantInfo);
            }
        }

        /// <summary>
        /// Syslog tH[}bg`̃^CX^vl <see cref="DateTime"/> ɕϊăCX^XɊi[܂B
        /// </summary>
        /// <param name="timeStamp">MMM dd HH:mm:ss `̓B</param>
        /// <exception cref="System.FormatException">^CX^ṽtH[}bg`܂B</exception>
        public void SetTimeStamp(string timeStamp)
        {
            if (timeStamp != null && timeStamp != "")
            {
                this.TimeStamp = DateTime.ParseExact(
                    timeStamp, "MMM d HH:mm:ss", DateTimeFormatInfo.InvariantInfo, DateTimeStyles.AllowWhiteSpaces);
            }
        }

        /// <summary>
        /// MSGp[gTAGCONTENTɐ؂蕪i[܂B
        /// </summary>
        /// <param name="msg">TAG  CONTENT ĂMSGp[g̕B</param>
        public void SetMSG(string msg)
        {
            if (msg != null && msg != String.Empty)
            {
                // [TAG][CONTENT] - TAG: ABNFp̂ / CONTENT: \ȕ
                Regex MSGRegex = new Regex(@"^(?>(?<TAG>[0-9a-zA-Z]{0,32}))(?<CONTENT>\P{Cn}*)", RegexOptionGroup);
                Match MSGMatch = MSGRegex.Match(msg);

                this.TAG = MSGMatch.Groups["TAG"].Value;
                this.CONTENT = MSGMatch.Groups["CONTENT"].Value;
            }
            else
            {
                this.TAG = String.Empty;
                this.CONTENT = String.Empty;
            }
        }

        /// <summary>
        /// <see cref="SyslogMessage" /> ftHg̕R[hŃGR[hoCgzԂ܂B
        /// </summary>
        /// <returns><see cref="SyslogMessage"/> ftHg̕R[hŃGR[hoCgzB</returns>
        public virtual byte[] ToByteArray()
        {
            return this.ToByteArray(null);
        }

        /// <summary>
        /// <see cref="SyslogMessage"/> w肳ꂽR[hŃGR[hoCgzƂĕԂ܂B
        /// </summary>
        /// <param name="encoding"><see cref="SyslogMessage"/> GR[h镶R[h\ <see cref="Encoding"/>B</param>
        /// <returns><see cref="SyslogMessage"/> w肵R[hŃGR[hoCgzB</returns>
        public virtual byte[] ToByteArray(Encoding encoding)
        {
            return this.ToByteArray(encoding, true);
        }

        /// <summary>
        /// <see cref="SyslogMessage"/> w肳ꂽR[hŃGR[hoCgzRFCɏŕԂ܂B
        /// </summary>
        /// <param name="encoding"><see cref="SyslogMessage"/> GR[h镶R[h\ <see cref="Encoding"/>B</param>
        /// <param name="lengthLimit">oCgz̍ő咷RFCŊ肳ꂽ1024oCgɐ܂B</param>
        /// <returns></returns>
        public virtual byte[] ToByteArray(Encoding encoding, bool lengthLimit)
        {
            Encoding Charset = encoding;

            // R[hw肳Ȃꍇ̓ftHg̕GR[h𗘗p
            if (Charset == null)
            {
                // ftHg̕GR[fBO𗘗p
                Charset = DefaultEncoding.GetDefaultEncoding();
            }

            // w肳ꂽR[h SyslogMessage oCgzɕϊ
            byte[] ByteArray = Charset.GetBytes(this.ToString());

            // oCgw肳ꂽꍇ
            if (lengthLimit)
            {
                int ByteCount = ByteArray.Length;

                // oCgz1024oCg𒴂Ăꍇ
                if (ByteCount > 1024)
                {
                    // oCgz̃TCY1024oCgɂ
                    ByteCount = 1024;

                    // w肳ꂽR[h SyslogMessage 𕶎z1024oCgȓŕϊ
                    char[] CharArray = Charset.GetChars(ByteArray, 0, ByteCount);

                    // zoCgzɕϊ
                    return Charset.GetBytes(CharArray);
                }
            }

            // oCgzԂB
            return ByteArray;
        }

        /// <summary>
        /// <see cref="SyslogMessage"/> ̕\B
        /// </summary>
        /// <returns><see cref="SyslogMessage"/> ̕\1024oCgł͂Ȃ1024܂łɕϊ܂B</returns>
        public override string ToString()
        {
            return this.ToString(true);
        }

        /// <summary>
        /// <see cref="SyslogMessage"/> ̕\B
        /// </summary>
        /// <param name="lengthLimit">̍ő咷1024ɐ܂B</param>
        /// <returns></returns>
        public virtual string ToString(bool lengthLimit)
        {
            StringBuilder SyslogBuilder = new StringBuilder(1024);

            // [PRI][HEADER][ ][MSG]
            SyslogBuilder.Append(this.PRI);
            SyslogBuilder.Append(this.HEADER);
            SyslogBuilder.Append(' ');
            SyslogBuilder.Append(this.MSG);

            // ̒1024𒴂ꍇ
            if (lengthLimit && SyslogBuilder.Length > 1024)
            {
                return SyslogBuilder.ToString(0, 1024);
            }
            else
            {
                return SyslogBuilder.ToString();
            }
        }

        #region ICloneable o
        /// <summary>
        /// <see cref="SyslogMessage"/> ̊SȃRs[쐬܂B
        /// </summary>
        /// <returns><see cref="SyslogMessage"/> ̊SȃRs[B</returns>
        object ICloneable.Clone()
        {
            return this.Clone();
        }

        /// <summary>
        /// <see cref="SyslogMessage"/> ̊SȃRs[쐬܂B
        /// </summary>
        /// <returns><see cref="SyslogMessage"/> ̊SȃRs[B</returns>
        public virtual SyslogMessage Clone()
        {
            // TODO:  SyslogMessage.Clone ǉ܂B
            return (SyslogMessage) this.MemberwiseClone();
        }
        #endregion

        /// <summary>
        /// zXg`FbN܂B
        /// </summary>
        /// <param name="host"></param>
        /// <returns></returns>
        private string LookupHostname(string host)
        {
            // zXgnꂽꍇ
            if (host != null && host != String.Empty)
            {
                try
                {
                    // IPAhXt
                    IPHostEntry HostEntry = Dns.GetHostByAddress(host);

                    // pŎn܂ĉpŏI63܂ł̃nCtƉpFQDN̍ŏ̃sIh܂ł̃zXg
                    Regex HostRegex = new Regex(@"^(?>(?<Host>[a-zA-Z]([-a-zA-Z0-9]{0,61}[a-zA-Z0-9])?))\.", RegexOptionGroup);
                    // zXg`FbN
                    Match HostMatch = HostRegex.Match(HostEntry.HostName);

                    // zXg̃`FbNɐꍇ
                    if (HostMatch.Success)
                    {
                        // zXgԂ
                        return HostMatch.Groups["Host"].Value;
                    } // tzXgsȏꍇ
                    else
                    {
                        // IPAhX̂܂ܕԂ
                        return host;
                    }
                } // IPAhXł͂ȂzXg܂܂Ă
                catch (FormatException)
                {
                    // pŎn܂ĉpŏI63܂ł̃nCtƉp̃zXg
                    Regex HostRegex = new Regex(@"^(?>(?<Host>[a-zA-Z]([-a-zA-Z0-9]{0,61}[a-zA-Z0-9])?))", RegexOptionGroup);
                    
                    if (HostRegex.IsMatch(host))
                    {
                        // zXgԂ
                        return host;
                    }
                    else
                    {
                        throw new ArgumentException("zXgɒ܂͗płȂ܂܂Ă܂B", "host");
                    }
                } // IPAhX̋tɎsꍇ
                catch (System.Net.Sockets.SocketException)
                {
                    // IPAhX̂܂ܕԂ
                    return host;
                }
            } // zXgnȂꍇ
            else
            {
                // [JzXg
                return Dns.GetHostName();
            }
        }
    }
}
