using System;
using System.Diagnostics;
using System.Text.RegularExpressions;

namespace Syslog
{
    /// <summary>
    /// Dx Facility ܂ Severity 𑊌ݕϊ܂B
    /// </summary>
    public class Priority
    {
        private const string FacilityRegexPattern
            = @"^(Kern(?:el)?)|(User(?:-level)?)|(Mail)|(Daemon)|(Auth)|(Syslog)|(Lpr)|(News)|(Uucp)|(Cron)|(AuthPriv)|(Ftp)|(Ntp)|(Security)|(Alert)|(Console)|(local[_-]?(?<level>[0-7]))$";

        private const string SeverityRegexPattern = @"^(Emerg(?:ency)?|Panic)|(Alert)|(Crit(?:ical)?)|(Err(?:or)?)|(Warn(?:ing)?)|(Notice)|(Info(?:rmation(?:al)?)?)|(Debug)$";

        private static Regex FacilityRegex = new Regex(FacilityRegexPattern, RegexOptions.IgnoreCase | RegexOptions.CultureInvariant);
        private static Regex SeverityRegex = new Regex(SeverityRegexPattern, RegexOptions.IgnoreCase | RegexOptions.CultureInvariant);

        /// <summary>
        /// 񂩂 <see cref="Facility"/> ɕϊ܂
        /// </summary>
        /// <param name="s"> <see cref="Facility"/> \B</param>
        /// <returns>ϊꂽ <see cref="Facility"/></returns>
        public static Facility ParseFacility(string s)
        {
            int FacilityLevel = Int32.MinValue;

            // 𐳋K\Ń}b`
            Match FacilityMatch = FacilityRegex.Match(s);

            // }b`K\̕\ʒu`FbN
            for (int i = 1; i < FacilityMatch.Groups.Count; i++)
            {
                // ݂̕\ʒũLv`}b`̂eXg
                Group g = FacilityMatch.Groups[i];

                // ݂̃Lv`}b`Ăꍇ
                if (g.Success)
                {
                    // [Jxł͂Ȃꍇ
                    if (i < 17)
                    {
                        // Lv`̈ʒu -1  Facility ƂĐݒ肷
                        FacilityLevel = i - 1;
                    }
                    else
                    {
                        // [Jx̏ꍇ͖̐lZ
                        FacilityLevel = i - 1 + Int32.Parse(FacilityMatch.Groups["level"].Value);
                    }

                    break;
                }
            }

            // Facility ǂݎȂꍇ
            if (FacilityLevel == Int32.MinValue)
            {
                Debug.Assert(false, "Facility słB", s);

                throw new ArgumentException("Facility słB", "s");
            }

            // Facility Ԃ
            return (Facility) FacilityLevel;
        }
        
        /// <summary>
        /// 񂩂 <see cref="Severity"/> ɕϊ܂
        /// </summary>
        /// <param name="s"> <see cref="Severity"/> \B</param>
        /// <returns>񂩂ϊꂽ <see cref="Severity"/>B</returns>
        public static Severity ParseSeverity(string s)
        {
            int SeverityLevel = Int32.MinValue;

            // 𐳋K\Ń}b`
            Match SeverityMatch = SeverityRegex.Match(s);

            // ǂ̃Lv`Ń}b`ׂ
            for (int i = 1; i < SeverityMatch.Groups.Count; i++)
            {
                Group g = SeverityMatch.Groups[i];

                // }b`Lv`ꍇ
                if (g.Success)
                {
                    // ̃Lv`̈ʒu Severity ɐݒ肷
                    SeverityLevel = i - 1;

                    break;
                }
            }

            // Severity ǂݎȂꍇ
            if (SeverityLevel == Int32.MinValue)
            {
                Debug.Assert(false, "Severity słB", s);

                throw new ArgumentException("Severity słB", "s");
            }

            // Severity Ԃ
            return (Severity) SeverityLevel;
        }

        /// <summary>
        /// <see cref="Facility"/> 𕶎\ɕϊ܂B
        /// </summary>
        /// <param name="facility">\ɕϊ <see cref="Facility"/>B</param>
        /// <returns><see cref="Facility"/> ̕\B</returns>
        public static string ToString(Facility facility)
        {
            return Enum.GetName(typeof(Facility), facility);
        }

        /// <summary>
        /// <see cref="Facility"/> 𕶎\ɕϊ܂B
        /// </summary>
        /// <param name="facility">\ɕϊ <see cref="Facility"/>B</param>
        /// <param name="verbose"><see cref="Facility"/> 𒷂\`ɕϊ邩w肵܂B</param>
        /// <returns><see cref="Facility"/> ̕\B</returns>
        public static string ToString(Facility facility, bool verbose)
        {
            // Z`w肳Ăꍇ
            if (!verbose)
            {
                // ʏ̕\`Ԃ
                return Priority.ToString(facility);
            }

            // ʕ\@\ɕϊĕ\LԂ
            return Enum.GetName(typeof(DisplayedFacility), facility);
        }

        /// <summary>
        /// <see cref="Severity"/> 𕶎\ɕϊ܂B
        /// </summary>
        /// <param name="severity"> <see cref="Severity"/> \B</param>
        /// <returns><see cref="Severity"/> ̕\B</returns>
        public static string ToString(Severity severity)
        {
            return Enum.GetName(typeof(Severity), severity);
        }

        /// <summary>
        /// <see cref="Severity"/> 𕶎\ɕϊ܂B
        /// </summary>
        /// <param name="severity"> <see cref="Severity"/> \B</param>
        /// <param name="verbose"><see cref="Severity"/> 𒷂\`ɕϊ邩w肵܂B</param>
        /// <returns><see cref="Severity"/> ̕\B</returns>
        public static string ToString(Severity severity, bool verbose)
        {
            // Z`w肳ꂽꍇ
            if (!verbose)
            {
                // ʏ̕\Ԃ
                return Priority.ToString(severity);
            }

            // ʕ\dxɕϊĕ\LԂ
            return Enum.GetName(typeof(DisplayedSeverity), severity);
        }
    }

    /// <summary>
    /// t@VeB
    /// </summary>
    public enum Facility
    {
        /// <summary>J[l bZ[W: 0</summary>
        Kern    = 0,

        /// <summary>[U x bZ[W: 1</summary>
        User    = 1,

        /// <summary>[ VXe: 2</summary>
        Mail    = 2,

        /// <summary>VXe f[: 3</summary>
        Daemon  = 3,

        /// <summary>ZLeBƔF: 4</summary>
        Auth    = 4,

        /// <summary>syslog bZ[W: 5</summary>
        Syslog  = 5,

        /// <summary>C v^: 6</summary>
        Lpr     = 6,

        /// <summary>lbg j[X: 7</summary>
        News    = 7,

        /// <summary>UUCP: 8</summary>
        Uucp    = 8,

        /// <summary>NbN f[: 9</summary>
        Cron    = 9,

        /// <summary>ZLeBƔF: 10</summary>
        AuthPriv = 10,

        /// <summary>t@C] (FTP): 11</summary>
        Ftp     = 11,

        /// <summary>킹 (NTP): 12</summary>
        Ntp     = 12,

        /// <summary>č: 13</summary>
        Security = 13,

        /// <summary>x: 14</summary>
        Alert   = 14,

        /// <summary>NbN f[: 15</summary>
        Console = 15,

        /// <summary>[J (0): 16</summary>
        Local_0 = 16,

        /// <summary>[J (1): 17</summary>
        Local_1 = 17,

        /// <summary>[J (2): 18</summary>
        Local_2 = 18,

        /// <summary>[J (3): 19</summary>
        Local_3 = 19,

        /// <summary>[J (4): 20</summary>
        Local_4 = 20,

        /// <summary>[J (5): 21</summary>
        Local_5 = 21,

        /// <summary>[J (6): 22</summary>
        Local_6 = 22,

        /// <summary>[J (7): 23</summary>
        Local_7 = 23
    }

    /// <summary>
    /// ʕ\t@VeB
    /// </summary>
    public enum DisplayedFacility
    {
        /// <summary>J[l bZ[W: 0</summary>
        KernelMessage       = 0,    // kernel messages

        /// <summary>[U x bZ[W: 1</summary>
        UserLevel           = 1,    // user-level messages

        /// <summary>[ VXe: 2</summary>
        MailSystem          = 2,    // mail system

        /// <summary>VXe f[: 3</summary>
        SystemDaemons       = 3,    // system daemons

        /// <summary>ZLeBƔF: 4</summary>
        SecurityAuthorization = 4,  // security/authorization messages

        /// <summary>syslog bZ[W: 5</summary>
        SyslogDaemon        = 5,    // messages generated internally by syslogd

        /// <summary>C v^: 6</summary>
        LPR                 = 6,    // line printer subsystem

        /// <summary>lbg j[X: 7</summary>
        NetworkNews         = 7,    // network news subsystem

        /// <summary>UUCP: 8</summary>
        UUCP                = 8,    // UUCP subsystem

        /// <summary>NbN f[: 9</summary>
        ClockDaemon         = 9,    // clock daemon

        /// <summary>ZLeBƔF: 10</summary>
        SecurityAuthorizationPrivate = 10,  // security/authorization private messages

        /// <summary>t@C] (FTP): 11</summary>
        FTP                 = 11,   // FTP daemon

        /// <summary>킹 (NTP): 12</summary>
        NTP                 = 12,   // NTP subsystem

        /// <summary>č: 13</summary>
        LogAudit            = 13,   // log audit

        /// <summary>x: 14</summary>
        LogAlert            = 14,   // log alert

        /// <summary>NbN f[: 15</summary>
        SystemConsole       = 15,   // System Console

        /// <summary>[J (0): 16</summary>
        Local_use_0         = 16,   // local use 0

        /// <summary>[J (1): 17</summary>
        Local_use_1         = 17,   // local use 1

        /// <summary>[J (2): 18</summary>
        Local_use_2         = 18,   // local use 2

        /// <summary>[J (3): 19</summary>
        Local_use_3         = 19,   // local use 3

        /// <summary>[J (4): 20</summary>
        Local_use_4         = 20,   // local use 4

        /// <summary>[J (5): 21</summary>
        Local_use_5         = 21,   // local use 5

        /// <summary>[J (6): 22</summary>
        Local_use_6         = 22,   // local use 6

        /// <summary>[J (7): 23</summary>
        Local_use_7         = 23    // local use 7
    }

    /// <summary>
    /// dx
    /// </summary>
    public enum Severity
    {
        /// <summary>vI: 0</summary>
        Emerg   = 0,

        /// <summary>x: 1</summary>
        Alert   = 1,

        /// <summary>@I: 2</summary>
        Crit    = 2,

        /// <summary>G[: 3</summary>
        Err     = 3,

        /// <summary>x: 4</summary>
        Warning = 4,

        /// <summary>ʒm: 5</summary>
        Notice  = 5,

        /// <summary>: 6</summary>
        Info    = 6,

        /// <summary>fobO: 7</summary>
        Debug   = 7
    }

    /// <summary>
    /// ʕ\dx
    /// </summary>
    public enum DisplayedSeverity
    {
        /// <summary>vI: 0</summary>
        Emergency = 0,
        /// <summary>x: 1</summary>
        Alert = 1,
        /// <summary>@I: 2</summary>
        Critical = 2,
        /// <summary>G[: 3</summary>
        Error = 3,
        /// <summary>x: 4</summary>
        Warning = 4,
        /// <summary>ʒm: 5</summary>
        Notice = 5,
        /// <summary>: 6</summary>
        Informational = 6,
        /// <summary>fobO: 7</summary>
        Debug = 7
    }

}