using System;
using System.Collections;
using System.Configuration;
using System.Diagnostics;
using System.Net;
using System.Text.RegularExpressions;
using System.Threading;
using Syslog.Config;

namespace Syslog.Listener
{
	/// <summary>
	/// ISyslogListener 𗘗pĂSyslogpPbgM Syslog Listener ControllerB
	/// </summary>
	public class ListenerController : IDisposable
    {
        /// <summary>syslogConfig/listener/modules</summary>
        private const string ConfigModuleSection = "syslogConfig/listener/modules";

        /// <summary>syslogConfig/listener/filters</summary>
        private const string ConfigFilterSection = "syslogConfig/listener/filters";

        /// <summary>Syslog̍őpPbgTCY</summary>
        private const int SyslogMaxPacketSize = 1024;

        /// <summary>񓯊ĂяopfQ[g</summary>
        private delegate void AsyncListenDelegate();

        /// <summary>IListener W[̃Xg</summary>
        private Hashtable Listeners = new Hashtable();

        /// <summary>IListenerFilter W[̃Xg</summary>
        private Hashtable ListenerFilterList = new Hashtable();

        /// <summary>MSyslogpPbg̃tB^OCxg</summary>
        private ListenerFilterEvent FilterEvent = new ListenerFilterEvent();

        /// <summary>MSyslogpPbg̃L[</summary>
        private Queue PacketQueue = new Queue(100);

        /// <summary>MCxgɓVOiOIuWFNg</summary>
        private AutoResetEvent ReceiveWait = new AutoResetEvent(false);

        /// <summary>T[rXꎞ~ԂɂȂĂ邩ǂʂtO</summary>
        private bool IsPause = false;

        /// <summary>sȃtH[}bg̃pPbg@ʂ <see cref="ErrorPacketRule"/>B</summary>
        private ErrorPacketRuleType ErrorPacketMethod = ErrorPacketRule.GetErrorPacketRule();

        /// <summary>pPbg̓e\ȕō\Ă邩`FbN鐳K\</summary>
        private Regex PrintablePacket = new Regex(@"\A(?>(?<Print>[\P{Cn}\P{Cc}]*))\z", RegexOptions.Compiled | RegexOptions.CultureInvariant | RegexOptions.ExplicitCapture | RegexOptions.Singleline);

        /// <summary>pPbg̓e\ȕ݂̂ɒu邽߂ׂĂ̈s\ȕɃ}b`鐳K\</summary>
        private Regex PrintableReplacing = new Regex(@"[\p{Cn}\p{Cc}]", RegexOptions.Compiled | RegexOptions.CultureInvariant | RegexOptions.ExplicitCapture | RegexOptions.Singleline);

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <exception cref="ModuleLoadingException">ISyslogListener W[܂ ISyslogListenerFilter tB^ W[̃[hɃG[ꍇɃX[܂B</exception>
		public ListenerController()
        {
            // tB^o^Xg擾
            AssemblyEntryCollection entries = (AssemblyEntryCollection) ConfigurationSettings.GetConfig(ConfigFilterSection);

            // tB^1o^ĂȂꍇ
            if (entries == null)
            {
                // ĝȂRNV쐬
                entries = new AssemblyEntryCollection(0);
            }

            // tB^o^
            foreach (AssemblyEntry entry in entries)
            {
                try
                {
                    // tB^̃CX^X쐬
                    ISyslogListenerFilter filter = (ISyslogListenerFilter) entry.CreateInstance();

                    Debug.Assert(filter != null, "Xi tB^̃[hɎs܂");

                    if (filter != null)
                    {
                        // tB^
                        filter.Init(FilterEvent, entry.Config);
                        // tB^o^
                        ListenerFilterList[entry.ID] = filter;
                    }
                    else
                    {
                        // OX[
                        throw new ModuleLoadingException(String.Format("[{0}] Xi W[̃[hɎs܂", entry.ID));
                    }
                }
                catch (Exception e)
                {
                    Debug.Assert(false, "Xi tB^̓ǂݍ݂Ɏs܂", e.ToString());

                    Trace.WriteLine("Xi tB^̓ǂݍ݂Ɏs܂!", "Syslog.Listener.ListenerController");
                    Trace.Indent();
                    Trace.WriteLine(e, "Syslog.Listener.ListenerController");
                    Trace.Unindent();

                    // OX[
                    throw e;
                }
            }

            // XiW[o^Xg擾
            AssemblyEntryCollection ListenerList = (AssemblyEntryCollection) ConfigurationSettings.GetConfig(ConfigModuleSection);

            // Xi1o^ĂȂꍇ
            if (ListenerList == null)
            {
                // xbZ[Wo
                Trace.WriteLine("Xio^Ă܂", "Syslog.Listener.ListenerController");

                // ĝȂRNV쐬
                ListenerList = new AssemblyEntryCollection(0);
            }

            // XiW[o^
            foreach (AssemblyEntry entry in ListenerList)
            {
                try
                {
                    // XĩCX^Xǉݒnč쐬
                    ISyslogListener Listener = (ISyslogListener) entry.CreateInstance(entry.Config);

                    Debug.Assert(Listener != null, "Xi W[̃[hɎs܂");

                    if (Listener != null)
                    {
                        // Xio^
                        this.AddListener(entry.ID, Listener);
                    }
                    else
                    {
                        // OX[
                        throw new ModuleLoadingException(String.Format("[{0}] Xi W[̃[hɎs܂", entry.ID));
                    }
                }
                catch (Exception e)
                {
                    Debug.Assert(false, "Xi W[̓ǂݍ݂Ɏs܂", e.ToString());

                    Trace.WriteLine("Xi W[̓ǂݍ݂Ɏs܂!", "Syslog.Listener.ListenerController");
                    Trace.Indent();
                    Trace.WriteLine(e, "Syslog.Listener.ListenerController");
                    Trace.Unindent();

                    // OX[
                    throw e;
                }
            }
		}

        /// <summary>
        /// o^ς݂̃Xi\[XJ܂B
        /// </summary>
        /// <remarks>
        /// ListenerController ɓo^ꂽ <see cref="Syslog.Listener.ISyslogListener"/> 
        /// 郊Xi <see cref="Syslog.Listener.ISyslogListener.Close"/> \bhĂяoA
        /// ׂĂ̓o^ς݃Xi CX^Xj\[XJ܂B
        /// </remarks>
        public void Close()
        {
            Dispose();
        }

        /// <summary>
        /// pPbgML[o܂B
        /// ML[ɃpPbgȂꍇ́ApPbgM܂ŃubN܂B
        /// </summary>
        /// <returns>MpPbgi[ <see cref="SyslogPacket"/> B</returns>
        public SyslogPacket GetPacket()
        {
            // pPbg擾܂őҋ@
            return this.GetPacket(-1);
        }

        /// <summary>
        /// pPbgML[o܂B
        /// ML[ɃpPbgȂꍇ́ApPbgM܂ŃubN܂B
        /// ̂Ƃ <paramref name="timeout"/> Ŏw肵~b̊ԂɃpPbgMȂꍇA
        /// pPbg̓҂ null QƂԂ܂B
        /// </summary>
        /// <param name="millisecondsTimeout">pPbg̓ҋ@~bB</param>
        /// <returns>MpPbgi[ <see cref="SyslogPacket"/> B ^CAEgꍇ null QƁB</returns>
        public SyslogPacket GetPacket(int millisecondsTimeout)
        {
            // w肵~bȓŃpPbg擾
            return this.GetPacket(new TimeSpan(0, 0, 0, 0, millisecondsTimeout));
        }

        /// <summary>
        /// pPbgML[o܂B
        /// ML[ɃpPbgȂꍇ́ApPbgM܂ŃubN܂B
        /// ̂Ƃ <paramref name="timeout"/> Ŏw肵ԊԊůԂɃpPbgMȂꍇA
        /// pPbg̓҂ null QƂԂ܂B
        /// </summary>
        /// <param name="timeout">pPbg̓ҋ@ <see cref="System.TimeSpan"/> B</param>
        /// <returns>MpPbgi[ <see cref="SyslogPacket"/> B ^CAEgꍇ null QƁB</returns>
        public SyslogPacket GetPacket(TimeSpan timeout)
        {
            // ꂽML[擾
            Queue SyncQueue = Queue.Synchronized(PacketQueue);

            SyslogPacket Packet = null;
            
            // L[ɃpPbgȂꍇ
            if (SyncQueue.Count < 1)
            {
                // VOiԂɂȂ܂őҋ@
                ReceiveWait.WaitOne(timeout, false);
            }

            // L[bNăt@gpPbgh~
            lock (SyncQueue.SyncRoot)
            {
                // L[ɃpPbgꍇ
                if (SyncQueue.Count > 0)
                {
                    // pPbg擾
                    Packet = (SyslogPacket) SyncQueue.Dequeue();
                }
            }

            // pPbgԂ
            return Packet;
        }

        /// <summary>
        /// <see cref="Syslog.Listener.ISyslogListener"/> 郊Xio^܂
        /// </summary>
        /// <param name="key">XiCX^XIDB</param>
        /// <param name="listener">o^郊Xi CX^XB</param>
        public void AddListener(object key, ISyslogListener listener)
        {
            // ܂Xio^ĂȂꍇ
            if (!Listeners.Contains(key))
            {
                Trace.WriteLine(String.Format("{0} XiŃXn߂܂", key), "Syslog.Listener.ListenerController");

                // XiXgɒǉ
                Listeners[key] = listener;

                // MCxg󂯎
                listener.PacketReceived += new ReceivedEventHandler(ISyslogListener_PacketReceived);

                // Xȋ҂󂯊Jn\bh񓯊IɌĂяo
                AsyncInvoke.AsyncDynamicInvoke(new AsyncListenDelegate(listener.Listen), null);
            }
        }

        /// <summary>
        /// <see cref="Syslog.Listener.ISyslogListener"/> 郊Xi폜܂
        /// </summary>
        /// <param name="key">XiCX^XIDB</param>
        /// <param name="listener">폜Xi CX^XB</param>
        public void RemoveListener(object key, ISyslogListener listener)
        {
            // Xgo^Ăꍇ
            if (Listeners.Contains(key))
            {
                Trace.WriteLine(String.Format("{0} XĩX𒆎~܂", key), "Syslog.Listener.ListenerController");

                // Xi CX^X擾
                ISyslogListener Listener = (ISyslogListener) Listeners[key];

                try
                {
                    // ҂󂯂𒆎~
                    Listener.Close();
                }
                catch (Exception e)
                {
                    Debug.Assert(false, "Xȋ҂󂯒~Ɏs܂", e.ToString());
                    
                    throw e;
                }
                finally
                {
                    // XiXg폜
                    Listeners.Remove(key);
                }
            }
        }

        /// <summary>
        /// Xiꎞ~ԂɈڍs܂B
        /// </summary>
        public void Pause()
        {
            Trace.WriteLine("Xiꎞ~܂", "Syslog.Listener.ListenerController");

            // tObN
            lock (this)
            {
                // łɈꎞ~ԂɂȂĂꍇ
                if (IsPause)
                {
                    // ݂̏Ԃێ
                    return;
                }
                // ܂ꎞ~ԂɂȂĂȂꍇ
                else
                {
                    // tO𗧂Ă
                    IsPause = true;
                }
            }

            // XiXgbN
            lock (Listeners.SyncRoot)
            {
                // XiXgɃXgꂽXiԂɕ
                foreach (ISyslogListener item in Listeners.Values)
                {
                    // Xi
                    item.Close();
                }
            }
        }

        /// <summary>
        /// XisԂɈڍs܂B
        /// </summary>
        public void Continue()
        {
            Trace.WriteLine("XiĊJ܂", "Syslog.Listener.ListenerController");

            // tObN
            lock (this)
            {
                // łɎsԂ̏ꍇ
                if (!IsPause)
                {
                    // s𑱂
                    return;
                }
                // ꎞ~Ԃ̏ꍇ
                else
                {
                    // tOɖ߂
                    IsPause = false;
                }
            }

            // XiXgbN
            lock (Listeners.SyncRoot)
            {
                // XiXgɃXgꂽXiԂɃI[v
                foreach (ISyslogListener item in Listeners.Values)
                {
                    // Xȋ҂󂯊Jn\bh񓯊IɌĂяo
                    AsyncInvoke.AsyncDynamicInvoke(new AsyncListenDelegate(item.Listen), null);
                }
            }
        }

        /// <summary>
        /// ubNĂM҂Xbh邽߂̃VOiOIuWFNgԂ܂B
        /// </summary>
        public AutoResetEvent WaitSignal
        {
            get
            {
                return ReceiveWait;
            }
        }

        #region IDisposable o
        /// <summary>
        /// ListenerController Ɠo^ς݂ <see cref="Syslog.Listener.ISyslogListener"/> \[XJ܂B
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
        }
        #endregion

        /// <summary>
        /// A}l[W \[XJ܂B
        /// </summary>
        /// <param name="disposing">ǉIɃ}l[W\[X܂B</param>
        protected void Dispose(bool disposing)
        {
            try
            {
                // XiXg̃XiԂɏ
                foreach (ISyslogListener item in Listeners.Values)
                {
                    try
                    {
                        // ҂󂯂𒆎~
                        item.Close();
                    }
                    catch (Exception e)
                    {
                        Debug.Assert(false, "Xi̎M~Ɏs܂", e.ToString());

                        Trace.WriteLine("Xi̎M~Ɏs܂!", "Syslog.Listener.ListenerController");
                        Trace.Indent();
                        Trace.WriteLine(e, "Syslog.Listener.ListenerController");
                        Trace.Unindent();

                        throw e;
                    }
                }
            }
            finally
            {
                // Xi XgNA
                Listeners.Clear();
            }

            try
            {
                // XitB^XgԂɏ
                foreach (ISyslogListenerFilter Filter in ListenerFilterList.Values)
                {
                    try
                    {
                        // XitB^̃\[X
                        Filter.Dispose();
                    }
                    catch (Exception e)
                    {
                        Debug.Assert(false, "Xi tB^̉Ɏs܂", e.ToString());

                        Trace.WriteLine("Xi tB^̉Ɏs܂!", "Syslog.Listener.ListenerController");
                        Trace.Indent();
                        Trace.WriteLine(e, "Syslog.Listener.ListenerController");
                        Trace.Unindent();

                        throw e;
                    }
                }
            }
            finally
            {
                // XitB^XgNA
                ListenerFilterList.Clear();
            }
        }

        /// <summary>
        /// ISyslogListner MSyslogpPbg܂B
        /// </summary>
        /// <param name="sender">SyslogpPbgM ISyslogListnerB</param>
        /// <param name="e">SyslogpPbg̃f[^ ReceivedEventArgsB</param>
        /// <returns>pPbg̏p\ bool lB true ͏ꂽƂ\A false ͏𒆎~Ƃ\B</returns>
        private bool ISyslogListener_PacketReceived(object sender, ReceivedEventArgs e)
        {
            // PreReceive tB^KpĖ߂l flase ̏ꍇ͌㑱̃pPbg𒆎~
            if (!FilterEvent.OnReceiving(e))
            {
                return false;
            }

            // pPbgTCYsȏꍇ͍\t@C̐ݒɂignore, drop, Reject肷

            // G[pPbg𖳎Ȃꍇ
            if (ErrorPacketMethod != ErrorPacketRuleType.Ignore)
            {
                SyslogPacket CheckPacket = e.Packet;
                // pPbgTCYKiǂ`FbN

                // pPbg̃f[^őTCY𒴂Ăꍇ
                if (CheckPacket.PacketData.LongLength > SyslogMaxPacketSize)
                {
                    // pPbĝ܂ܔj悤Ɏw肳Ăꍇ
                    if (ErrorPacketMethod == ErrorPacketRuleType.Drop)
                    {
                        // pPbg̏𒆎~
                        return false;
                    }

                    // G[bZ[WpPbg\z
                    SyslogMessage ErrorPacketMessage = new SyslogMessage();

                    ErrorPacketMessage.Facility = (int) Facility.Syslog;
                    ErrorPacketMessage.Severity = (int) Severity.Warning;
                    ErrorPacketMessage.Hostname = Dns.GetHostName();
                    ErrorPacketMessage.SetMSG(
                        String.Format("PacketSizeError: {0}, {1}, {2}", 
                        CheckPacket.ReceivedDateTime.ToString("s"), 
                        CheckPacket.RemoteEndPoint.ToString(), 
                        CheckPacket.LocalEndPoint.ToString()));

                    // ftHg̕R[hŃG[bZ[WGR[hăG[bZ[W쐬
                    CheckPacket.PacketData = ErrorPacketMessage.ToByteArray(new SyslogPacket().Encode);
                    // IWĩpPbgMIvVlƂċL^
                    CheckPacket.Option.Add("OriginalSourceAddress", CheckPacket.RemoteEndPoint.ToString());
                    // pPbg̑M[JzXgɕύX
                    CheckPacket.RemoteEndPoint = new IPEndPoint(Dns.Resolve(Dns.GetHostName()).AddressList[0], CheckPacket.RemoteEndPoint.Port);
                }
                    // pPbg̓e󎚉\ȕ񂾂ō\Ă邩`FbN
                    // 󎚉\ȕȊO̕񂪊܂܂Ăꍇ
                else if (!PrintablePacket.IsMatch(CheckPacket.GetPacketString()))
                {
                    // pPbĝ܂ܔj悤Ɏw肳Ăꍇ
                    if (ErrorPacketMethod == ErrorPacketRuleType.Drop)
                    {
                        // pPbg̏𒆎~
                        return false;
                    }

                    // G[bZ[WpPbg\z
                    SyslogMessage ErrorPacketMessage = new SyslogMessage();

                    ErrorPacketMessage.Facility = (int) Facility.Syslog;
                    ErrorPacketMessage.Severity = (int) Severity.Warning;
                    ErrorPacketMessage.Hostname = Dns.GetHostName();
                    ErrorPacketMessage.SetMSG(
                        String.Format("PacketCharCodeError: {0}, {1}, {2}", 
                        CheckPacket.ReceivedDateTime.ToString("s"), 
                        CheckPacket.RemoteEndPoint.ToString(), 
                        CheckPacket.LocalEndPoint.ToString()));

                    // ftHg̕R[hŃG[bZ[WGR[hăG[bZ[W쐬
                    CheckPacket.PacketData = ErrorPacketMessage.ToByteArray(new SyslogPacket().Encode);
                    // IWĩpPbgMIvVlƂċL^
                    CheckPacket.Option.Add("OriginalSourceAddress", CheckPacket.RemoteEndPoint.ToString());
                    // pPbg̑M[JzXgɕύX
                    CheckPacket.RemoteEndPoint = new IPEndPoint(Dns.Resolve(Dns.GetHostName()).AddressList[0], CheckPacket.RemoteEndPoint.Port);
                }
            }
            // G[pPbg𖳎ꍇ
            else
            {
                // s\ȕ폜ăoCizɐݒ肵Ȃ
                e.Packet.SetPacketString(PrintableReplacing.Replace(e.Packet.GetPacketString(), String.Empty));
            }

            // PreReceive tB^KpĖ߂l false ̏ꍇ͌㑱̃pPbg𒆎~
            if (!FilterEvent.OnReceived(e))
            {
                return false;
            }

            // ꂽQueuẽbp[擾
            Queue SyncQueue = Queue.Synchronized(PacketQueue);

            // ReceivePacket IuWFNgL[ɒǉ
            SyncQueue.Enqueue(e.Packet);

            // VOi𑗂ăCXbhŎMpPbg̏Jn
            ReceiveWait.Set();
            // ̃Xbhɏ
            Thread.Sleep(0);

            return true;
        }
    }
}
