using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.IO.IsolatedStorage;
using System.Text;
using System.Timers;
using System.Net;
using System.Xml;
using Syslog.Config.EventLog;
using Syslog.Config.EventLog.SectionHandler;

namespace Syslog.Listener
{
    #region SimpleEventLogListener
	/// <summary>
	/// CxgO̕ύX|[OĒǉꂽCxgGg荞݂܂B
	/// </summary>
	/// <remarks>
	/// <para>CxgO|[OČoVCxgGgA
    /// K\ƃ}b`ēK؂ȗDxɐݒ肵 Syslog ƂĎ荞݂܂B</para>
    /// <para>eventlog Gg</para>
    /// <list type="table">
    ///     <listheader>
    ///         <term></term>
    ///         <description>ݒl</description>
    ///     </listheader>
    ///     <item>
    ///         <term>target</term>
    ///         <description>|[OsCxgOw肵܂B
    ///             <para>1̃W[ CX^Xŕ̃O𓯎Ƀ|[O邱Ƃ͂ł܂B
    ///             ̃O|[OƂIDlύX̃W[ CX^XǉĂB</para></description>
    ///     </item>
    ///     <item>
    ///         <term>machine</term>
    ///         <description>ڑCxgÕRs[^w肵܂B[J}V̏ꍇ "." (sIh)B</description>
    ///     </item>
    ///     <item>
    ///         <term>interval</term>
    ///         <description>|[OsԊuB1bȏ̃~bPʂŎw肵܂B</description>
    ///     </item>
    /// </list>
    /// <para>priority Gg</para>
    /// <list type="table">
    ///     <listheader>
    ///         <term></term>
    ///         <description>ݒl</description>
    ///     </listheader>
    ///     <item>
    ///         <term>facility</term>
    ///         <description>}b`ꍇɐݒ肷 FacilityB</description>
    ///     </item>
    ///     <item>
    ///         <term>severity</term>
    ///         <description>}b`ꍇɐݒ肷 SeverityB</description>
    ///     </item>
    /// </list>
    /// <para>regex Gg</para>
    /// <list type="table">
    ///     <listheader>
    ///         <term></term>
    ///         <description>ݒl</description>
    ///     </listheader>
    ///     <item>
    ///         <term>source</term>
    ///         <description>K\}b`f[^B
    ///         <para>EventID     = CxgIDB</para>
    ///         <para>EntryType   = Cxg̎ށB
    ///         <code>{Error|Warning|Information|FailureAudit|SuccessAudit}</code></para>
    ///         <para>Source      = Cxg𔭐AvP[VB</para>
    ///         <para>UserName    = Cxg𔭐[UB</para>
    ///         <para>HostName    = Cxg𔭐Rs[^B</para>
    ///         <para>DateTime    = CxgB</para>
    ///         <para>CategoryID  = GgɊ֘AtăJeSԍB</para>
    ///         <para>Category    = GgɊ֘AtꂽJeSeLXgB</para>
    ///         <para>Message     = Gg̃bZ[WeB</para></description>
    ///     </item>
    ///     <item>
    ///         <term>eLXg m[h</term>
    ///         <description>CxgOGgƃ}b`鐳K\B
    ///         <para>啶Ə̋ʂȂBGXP[vȂXy[X͖BsIhׂ͂Ă̕Ƀ}b`B</para></description>
    ///     </item>
    /// </list>
    /// <para>CxgOɒʎZ 2,147,483,647 𒴂Cxg܂ꂽ͕̓słB</para>
    /// </remarks>
	/// <example>
	/// <para>\t@C̋LqB</para>
	/// <code>
    /// &lt;eventlog target="Application|Security|System|..." machine="." interval="10000"&gt;
    ///   &lt;priority facility="Kern|User|..." severity="Emerg|Alert|Crit|..."&gt;
    ///     &lt;regex source="EventID|EntryType|Source|UserName|HostName|DateTime|CategoryID|Category|Message"&gt;
    ///       {RegexPattern}
    ///     &lt;/regex&gt;
    ///     &lt;regex source="..." /&gt;
    ///     ...
    ///   &lt;/priority&gt;
    ///   ...
    /// &lt;/eventlog&gt;</code>
    /// </example>
    #endregion
	public class SimpleEventLogListener : ISyslogListener
	{
        private const string EventLogSettingKey = "eventlog";
        private const string TargetEventLogNameKey = "target";
        private const string TargetMachineNameKey = "machine";
        private const string PollingIntervalKey = "interval";
        private const string DefaultLogName = "Application";
        private const string DefaultMachineName = ".";
        private const string ListenerDisplayName = "Syslog Simple Windows EventLog Listener";
        private const int LocalHostPort = 514;
        private const int MinimumInterval = 1000;

        /// <summary>
        /// Dx@w肳ꂽ <see cref="EventLogPriorityEntry"/> ̔zB
        /// </summary>
        protected ArrayList PriorityList = null;
        private Timer PollingTimer = new Timer(5000);
        private DateTime StopedTime = DateTime.MaxValue;
        private EventLog eventLog = null;
        private DateTime eventLatestTime = DateTime.Now;
        private int eventLatestIndex = 0;

        /// <summary>
        /// Syslog pPbgRg[ɓfQ[g
        /// </summary>
        public event Syslog.Listener.ReceivedEventHandler PacketReceived;

        /// <summary>
        /// RXgN^B
        /// </summary>
        /// <param name="setting">\t@Cݒ蕶B</param>
		public SimpleEventLogListener(string setting)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(setting);

            XmlNode eventlogNode = null;

            // ׂĂ̎qm[h𒲂ׂ
            foreach (XmlNode node in doc.ChildNodes)
            {
                // Gg m[hŃ[JݒGg̏ꍇ
                if (node.NodeType == XmlNodeType.Element && node.LocalName == EventLogSettingKey)
                {
                    // m[h擾
                    eventlogNode = node;

                    break;
                }
            }

            // XiŜ̃JX^ݒ擾
            this.InitializeCustomConfig(eventlogNode);

            // Dx肷ݒlXg擾
            PriorityList = (ArrayList) new EventLogPriorityListSectionHandler().Create(doc, null, eventlogNode);

            // ^C}Cxgݒ肷
            PollingTimer.Elapsed += new ElapsedEventHandler(this.OnTimedEvent);

            // ^C}̋NԊuŒԊuȉ̏ꍇ
            if (PollingTimer.Interval < MinimumInterval)
            {
                // NԊuŒԊuɐݒ
                PollingTimer.Interval = MinimumInterval;
            }
        }

        #region ISyslogListener o
        /// <summary>
        /// Cxg̃|[OJn܂B
        /// </summary>
        public void Listen()
        {
            /// TODO: T[rXJnÕO擾ł悤ɂ
            /// TODO: Application CxgÕT[rXNbZ[W𗘗p
            /// 
            /// TODO: T[rẌꎞ~/ĊJ̃G[

            EventLogEntry entry = eventLog.Entries[eventLog.Entries.Count - 1];

            // ŐṼGg̃CxgCfbNX擾
            eventLatestIndex = entry.Index;
            eventLatestTime = entry.TimeWritten;

            // ^C}CxgJn
            PollingTimer.Start();
        }

        /// <summary>
        /// Cxg̃|[OI܂B̃\bh Dispose() Ăяo܂B
        /// </summary>
        public void Close()
        {
            this.Dispose();
        }

        /// <summary>
        /// SimpleEventLogListener ̃thłB
        /// </summary>
        public string Name
        {
            get
            {
                return ListenerDisplayName;
            }
        }

        /// <summary>
        /// Cxg̃Õ|[OsԂ܂B
        /// </summary>
        public bool IsListening
        {
            get
            {
                return PollingTimer.Enabled;
            }
        }
        #endregion

        #region IDisposable o
        /// <summary>
        /// CX^X̕ێĂ郊\[X܂B
        /// </summary>
        public void Dispose()
        {
            // IuWFNgbN
            lock (this)
            {
                // ^C}~߂
                PollingTimer.Stop();
                // ^C}̃\[X
                PollingTimer.Dispose();

                // ^C}~߂L^
                StopedTime = DateTime.Now;
            }

            if (eventLog != null)
            {
                // CxgÕ\[X
                eventLog.Dispose();
            }
        }
        #endregion

        /// <summary>
        /// <see cref="EventLogEntry"/>  <see cref="SyslogPacket"/> ɕϊ܂B
        /// </summary>
        /// <param name="entry"><see cref="SyslogPacket"/> ֕ϊ <see cref="EventLogEntry"/>B</param>
        /// <returns>ϊꂽ <see cref="SyslogPacket"/>B</returns>
        protected SyslogPacket Transform(EventLogEntry entry)
        {
            SyslogPacket packet = new SyslogPacket();
            packet.LocalEndPoint = this.GetLocalEndPoint();
            // pPbg̑M[JzXgɂ
            packet.RemoteEndPoint = this.GetLocalEndPoint();
            // IvVlɃbZ[W̏o𖾋L
            packet.Option.Add("SimpleEventLogListener", String.Format("Message from Windows EventLog - [{0}].", eventLog.LogDisplayName));

            SyslogMessage msg = new SyslogMessage();
            msg.TimeStamp = entry.TimeGenerated;
            msg.Hostname = entry.MachineName;

            // Dxݒ肷
            this.SetPriority(msg, entry);

            try
            {
                // bZ[W{ݒ肷
                msg.SetMSG(String.Format("{0}: {1}", entry.Source, entry.Message));
            }
            catch
            {
            }

            // ftHg̕R[hŃoCif[^ɕϊ
            packet.PacketData = msg.ToByteArray();

            // ϊꂽ SyslogPacket Ԃ
            return packet;
        }

        /// <summary>
        /// \t@Cɂ <see cref="EventLogEntry"/> ɓK؂ Priority 蓖Ă܂B
        /// </summary>
        /// <param name="msg">Priority 蓖Ă <see cref="SyslogPacket"/>B</param>
        /// <param name="entry">Priority 蓖Ă邽߂̏ <see cref="EventLogEntry"/>B</param>
        protected void SetPriority(SyslogMessage msg, EventLogEntry entry)
        {
            bool RegexComplete = true;

            // ׂĂ̗Dx\Xg܂
            foreach (EventLogPriorityEntry priority in PriorityList)
            {
                // }b`OŃtO𗧂Ă
                RegexComplete = true;

                // ׂĂ̐K\}b`܂
                foreach (EventLogRegexEntry regex in priority.Regexes)
                {
                    // K\}b`Ȃꍇ
                    if (!regex.IsMatch(entry))
                    {
                        // ȂƂtOɋL^
                        RegexComplete = false;

                        // }b`𒆎~
                        break;
                    }
                }

                // K\O[vׂĂɃ}b`ꍇ
                if (RegexComplete)
                {
                    // @\蓖Ă
                    msg.Facility = (int) Priority.ParseFacility(priority.Facility);
                    // dx蓖Ă
                    msg.Severity = (int) Priority.ParseSeverity(priority.Severity);

                    // x}b`c̃Xg͖
                    return;
                }
            }

            // ftHg̋@\蓖Ă
            msg.Facility = (int) Facility.User;

            // ftHg̏dx蓖Ă
            switch (entry.EntryType)
            {
                case EventLogEntryType.Error:
                {
                    msg.Severity = (int) Severity.Err;

                    break;
                }
                case EventLogEntryType.Warning:
                {
                    msg.Severity = (int) Severity.Warning;

                    break;
                }
                case EventLogEntryType.Information:
                {
                    msg.Severity = (int) Severity.Info;

                    break;
                }
                case EventLogEntryType.FailureAudit:
                {
                    msg.Facility = (int) Facility.Auth;
                    msg.Severity = (int) Severity.Alert;

                    break;
                }
                case EventLogEntryType.SuccessAudit:
                {
                    msg.Facility = (int) Facility.Auth;
                    msg.Severity = (int) Severity.Notice;

                    break;
                }
            }
        }

        /// <summary>
        /// ǂݍ񂾃CxgOGg Syslog Ƃď܂B
        /// </summary>
        /// <param name="e">Syslog pPbgƊ֘Af[^܂ <see cref="ReceivedEventArgs"/>B</param>
        /// <returns>pPbg̏p邩肵܂B</returns>
        protected bool OnPacketReceived(ReceivedEventArgs e)
        {
            // CxgfQ[gƃCxgmF܂
            if (PacketReceived != null && e != null)
            {
                // MCxg𔭐܂
                return PacketReceived(this, e);
            }
            // CxgfQ[ǧĂяoĂȂꍇ
            else
            {
                // 𒆎~܂B
                return false;
            }
        }

        /// <summary>
        /// w肵C^[oƂɋNăCxgO`FbN܂B
        /// </summary>
        /// <param name="source">Cxg̔B</param>
        /// <param name="e">^C}CxgŗL <see cref="ElapsedEventArgs"/>B</param>
        private void OnTimedEvent(object source, ElapsedEventArgs e)
        {
            // IuWFNgbN
            lock (this)
            {
                // ^C}~߂CxgԂꍇ
                if (StopedTime <= e.SignalTime)
                {
                    // 𒆎~
                    return;
                }
            }

            // CxgOGg̃RNV擾
            EventLogEntryCollection EventEntry = eventLog.Entries;

            // ŌɏCxgԌÂCxgGgVꍇ
            if (eventLatestTime < EventEntry[0].TimeWritten)
            {
                // ׂẴO
                foreach (EventLogEntry entry in EventEntry)
                {
                    this.ReadEntry(entry);
                }

                return;
            }

            // ς݃GĝЂƂ̃RNVCfbNXvZ
            int NextEntryIndex = eventLatestIndex - EventEntry[0].Index + 1;

            // RNVCfbNX}CiXɂȂꍇ
            if (NextEntryIndex < 0)
            {
                Trace.WriteLine(String.Format("CxgOGg Index }CiXɂȂĂ܂, {0}", NextEntryIndex));

                // őLlZ
                NextEntryIndex += Int32.MaxValue;
            }

            // CxgO珈ĂȂGgǂݎďs
            for (int i = NextEntryIndex; i < EventEntry.Count; i++)
            {
                this.ReadEntry(EventEntry[i]);
            }
        }

        /// <summary>
        /// CxgOGgǂݎϊ Syslog Ƃď܂B
        /// </summary>
        /// <param name="entry"></param>
        private void ReadEntry(EventLogEntry entry)
        {
            // ŌɏCxgCfbNXXV
            eventLatestIndex = entry.Index;
            // Ō̏Cxg̏ݎXV
            eventLatestTime = entry.TimeWritten;

            // CxgOϊ Syslog ƂăCxg𔭐ď܂
            this.OnPacketReceived(new ReceivedEventArgs(this, this.Transform(entry)));
        }

        /// <summary>
        /// [JzXg <see cref="IPEndPoint"/> Ԃ܂B
        /// </summary>
        /// <returns>[JzXg킷 <see cref="IPEndPoint"/>B</returns>
        private IPEndPoint GetLocalEndPoint()
        {
            IPEndPoint LocalEndPoint = null;
            
            try
            {
                // [JzXgtă[JzXg IPEndPoint 擾
                LocalEndPoint = new IPEndPoint(Dns.Resolve(Dns.GetHostName()).AddressList[0], LocalHostPort);
            }
            // tłȂꍇ
            catch
            {
                // [vobNAhXw肷
                LocalEndPoint = new IPEndPoint(IPAddress.Loopback, LocalHostPort);
            }

            // IPEndPoint Ԃ
            return LocalEndPoint;
        }

        /// <summary>
        /// SimpleEventLogListener ̓肷JX^\ݒ肵܂
        /// </summary>
        /// <param name="setting">JX^\Lq <see cref="XmlNode"/>B</param>
        private void InitializeCustomConfig(XmlNode setting)
        {
            // [JJX^ݒGgł͂Ȃꍇ
            if (setting.LocalName != EventLogSettingKey)
            {
                // 𒆎~
                return;
            }

            // ftHg̃O擾
            string LogName = DefaultLogName;
            // ftHg̃Rs[^擾
            string MachineName = DefaultMachineName;

            // m[h̃Agr[g擾
            XmlAttributeCollection attr = setting.Attributes;

            // Ow肳Ăꍇ
            if (attr[TargetEventLogNameKey] != null)
            {
                // O擾
                LogName = attr[TargetEventLogNameKey].Value;
            }

            // Rs[^w肳Ăꍇ
            if (attr[TargetMachineNameKey] != null)
            {
                // Rs[^擾
                MachineName = attr[TargetMachineNameKey].Value;
            }

            // w肳CxgO擾
            eventLog = new EventLog(LogName, MachineName);

            // v[OC^[ow肳Ăꍇ
            if (attr[PollingIntervalKey] != null)
            {
                try
                {
                    // v[OC^[oݒ肷
                    PollingTimer.Interval = double.Parse(attr[PollingIntervalKey].Value);
                }
                catch
                {
                }
            }
        }
    }
}
