using System;
using System.Collections;
using System.Diagnostics;
using System.Net;
using System.Net.Sockets;
using System.Text;
using Syslog.Config.SectionHandler;

namespace Syslog.Listener
{
	/// <summary>
	/// \Pbg C^[tFCXJ UDP pPbgM܂B
	/// </summary>
    public class UdpListener : ISyslogListener
    {
        private bool IsListened = false;
        private bool disposed = false;
        private IList sockets = new ArrayList();
        private IList endPointList = new ArrayList();

        /// <summary>
        /// RXgN^BApp.config [U\pPbg҂󂯂\Pbg肵܂B
        /// </summary>
        /// <param name="setting">App.config JX^RtBO[Vm[hB</param>
        public UdpListener(string setting)
        {
            // ҂󂯃|[gXg擾
            endPointList = (IList) IPAddressListSectionHandler.Parse(setting);
        }

        /// <summary>
        /// fXgN^B
        /// </summary>
        ~UdpListener()
        {
            // A}l[W\[Xj
            this.Dispose(false);
        }

        /// <summary>
        /// \Pbg̔񓯊Ms܂B
        /// </summary>
        /// <param name="ar">񓯊̃Xe[^X𑀍삷 IAsyncResultB</param>
        protected void ReceiveFromCallback(IAsyncResult ar)
        {
            // pPbgML^
            DateTime ReceiveTime = DateTime.Now;

            // \[XłɔjĂꍇ͎M~
            if (this.Disposed)
                return;

            // IAsyncResult  ListenerState IuWFNg擾
            ListenerState state = (ListenerState) ar.AsyncState;
            // ListenerState  \Pbg擾
            Socket socket = state.Sockets;

            // M\[XIPAhXi[IPAddressIuWFNg
            IPAddress AcceptAddress = null;

            // IPv6x[X̃\Pbg̏ꍇ
            if (state.Sockets.AddressFamily == AddressFamily.InterNetworkV6)
            {
                // ׂIPv6AhX󂯓
                AcceptAddress = IPAddress.IPv6Any;
            }
            // IPv6ȊÕ\Pbg̏ꍇ
            else
            {
                // ׂĂIPv4AhX󂯓
                AcceptAddress = IPAddress.Any;
            }

            // IPAddressMEndPoint쐬
            EndPoint remoteEP = (EndPoint) new IPEndPoint(AcceptAddress, 0);

            // Mf[^oCg
            int ReadBytes = 0;

            try
            {
                // pPbg̎Mf[^oCg擾
                ReadBytes = socket.EndReceiveFrom(ar, ref remoteEP);
            }
            catch (ObjectDisposedException)
            {
                // \[XłɔjĂꍇ͎M~
                return;
            }
            catch (SocketException e)
            {
                Console.WriteLine(e.ToString());
            }

            // Mf[^ꍇ
            if (ReadBytes > 0)
            {
                // Mf[^i[ SyslogPacket 쐬
                SyslogPacket Packet = new SyslogPacket();

                // obt@MpPbg̃f[^oCgɃobt@\z
                byte[] PacketData = new byte[ReadBytes];
                // Xe[gIuWFNg̃obt@Mf[^̃oCgǂݍ
                Array.Copy(state.Buffer, 0, PacketData, 0, ReadBytes);

                // Mf[^ SyslogPacket Ɋi[
                Packet.PacketData = PacketData;
                // pPbg̎MJni[
                Packet.ReceivedDateTime = ReceiveTime;
                // pPbgM[JC^[tFCXAhXi[
                Packet.LocalEndPoint = (IPEndPoint) socket.LocalEndPoint;
                // pPbg𑗐M\[XAhXi[
                Packet.RemoteEndPoint = (IPEndPoint) remoteEP;

                // pPbg̎MCxg𔭐
                this.OnPacketReceived(new ReceivedEventArgs(this, Packet));

                // V ListenerState IuWFNg쐬
                ListenerState ReceiveState = new ListenerState(socket.AddressFamily);
                // ListenerState Ƀ\Pbg֘At
                ReceiveState.Sockets = socket;

                // \[XłɔjĂꍇ͎M~
                if (this.Disposed)
                    return;

                try
                {
                    // ̃pPbg҂󂯂
                    socket.BeginReceiveFrom(ReceiveState.Buffer, 0, ReceiveState.BufferSize, SocketFlags.None,
                        ref ReceiveState.TempEndPoint, new AsyncCallback(this.ReceiveFromCallback), ReceiveState);
                }
                catch (ObjectDisposedException)
                {
                    // \[XłɔjĂꍇ͎M~
                    return;
                }
            }
        }

        /// <summary>
        /// A}l[Wh\[X܂B
        /// </summary>
        /// <param name="disposing">ǉIɃ}l[W\[X܂B</param>
        protected void Dispose(bool disposing)
        {
            try
            {
                // \[X܂jĂȂꍇ
                if (!this.Disposed)
                {
                    // \PbgXgbN܂
                    lock (sockets.SyncRoot)
                    {
                        // \PbgXg̃\PbgԂɏ܂
                        foreach (object item in sockets)
                        {
                            // \Pbgi[Ă邱ƂmF
                            if (item != null)
                            {
                                // \Pbg
                                Socket socket = (Socket) item;

                                Trace.WriteLine(String.Format("{0} ̃\PbgĂ܂...", socket.LocalEndPoint.ToString()), "Syslog.Listener.UdpListener");

                                socket.Close();
                            }
                        }

                        // \PbgXgNA
                        sockets.Clear();
                    }
                }
            }
            finally
            {
                // \[X̔jƂL^
                this.Disposed = true;

                // ҂󂯂~ƂL^
                this.IsListen = false;
            }
        }

        /// <summary>
        /// CX^X̕ێĂ郊\[XĂ邩\܂B
        /// </summary>
        private bool Disposed
        {
            get
            {
                // CX^XbN܂
                lock (this)
                {
                    return disposed;
                }
            }
            set
            {
                // CX^XbN܂
                lock (this)
                {
                    disposed = value;
                }
            }
        }

        /// <summary>
        /// \PbgJăpPbg҂󂯂Ă邩ǂ\܂B
        /// </summary>
        private bool IsListen
        {
            get
            {
                // CX^XbN܂
                lock (this)
                {
                    return this.IsListened;
                }
            }
            set
            {
                // CX^XbN܂
                lock (this)
                {
                    this.IsListened = value;
                }
            }
        }

        #region IListener o
        /// <summary>
        /// pPbgMɔCxgB
        /// </summary>
        public event ReceivedEventHandler PacketReceived;

        /// <summary>
        /// \PbgJăpPbg̑҂󂯂Jn܂B
        /// </summary>
        public void Listen()
        {
            Trace.WriteLine("\PbgJĂ܂...", "Syslog.Listener.UdpListener");

            // ȑO Dispose() \bhĂяoĂꍇ
            if (this.Disposed)
            {
                // \[X̔jłȂƂL^
                this.Disposed = false;

                // t@CiCǓĂяoēo^
                GC.ReRegisterForFinalize(this);
            }

            // RFC3484 - IPv6 \[XAhXI?

            Trace.Indent();

            // pPbg҂󂯂郍[JAhXƃ|[gXg擾
            foreach (IPEndPoint LocalEP in endPointList)
            {
                Trace.WriteLine(String.Format("{0} Ń\PbgJĂ܂", LocalEP.ToString()), "Syslog.Listener.UdpListener");

                // \Pbg쐬
                Socket socket = new Socket(LocalEP.AddressFamily, SocketType.Dgram, ProtocolType.Udp);

                lock (sockets.SyncRoot)
                {
                    // \PbgXgɃ\Pbgǉ
                    sockets.Add(socket);
                }
            
                try
                {
                    // Xe[gIuWFNg
                    ListenerState state = new ListenerState(socket.AddressFamily);
                    // Xe[gIuWFNgɃ\Pbgi[
                    state.Sockets = socket;

                    // \PbgJ
                    socket.Bind((EndPoint) LocalEP);

                    // pPbg̑҂󂯂Jn
                    socket.BeginReceiveFrom(state.Buffer, 0, state.BufferSize, SocketFlags.None,
                        ref state.TempEndPoint, new AsyncCallback(this.ReceiveFromCallback), state);

                    Trace.WriteLine(String.Format("{0} Ń\PbgJ܂", LocalEP.ToString()), "Syslog.Listener.UdpListener");
                }
                catch (SocketException e)
                {
                    Debug.Assert(false, "\Pbg̃I[vɃG[܂", e.ToString());

                    Trace.WriteLine("\Pbg̃I[vɃG[܂", "Syslog.Listener.UdpListener");
                    Trace.Indent();
                    Trace.WriteLine(e, "Syslog.Listener.UdpListener");
                    Trace.Unindent();

                    // łɃ\Pbgꕔ쐬Ăꍇ
                    if (socket != null)
                    {
                        // \Pbgmɕ܂
                        socket.Close();
                    }

                    // Ô܂܃X[܂
                    throw e;
                }
            }

            Trace.Unindent();
            Trace.WriteLine("ׂẴ\PbgJ܂", "Syslog.Listener.UdpListener");

            // ҂󂯂JnƂL^B
            this.IsListen = true;
        }

        /// <summary>
        /// \Pbgă\[X܂B
        /// </summary>
        public void Close()
        {
            // \[Xj
            this.Dispose();
        }

        /// <summary>
        /// pPbgMƂɃCxg𔭐܂B
        /// </summary>
        /// <param name="e">MpPbgƊ֌Wf[^i[ <see cref="Syslog.Listener.ReceivedEventArgs"/> CX^XB</param>
        /// <returns>pPbg̏p邩肵܂B</returns>
        protected bool OnPacketReceived(ReceivedEventArgs e)
        {
            // Received CxgfQ[gƃCxgf[^mFăCxg𔭐܂
            if (PacketReceived != null && e != null)
            {
                // Cxg𔭐߂lԂ܂
                return PacketReceived(this, e);
            }
            // Received CxgfQ[gĂяoȂꍇ
            else
            {
                // ̒~\lԂ܂
                return false;
            }
        }

        /// <summary>
        /// UdpListener ̃thłB
        /// </summary>
        public string Name
        {
            get
            {
                // Xĩthi[StringBuilderł
                StringBuilder ListenerName = new StringBuilder(128);
                // {̃thw肵܂
                ListenerName.Append("Syslog UDP Listener");

                // \PbgXgɃ\Pbg݂ꍇ
                if (sockets.Count > 0)
                {
                    // \PbgXgbN܂
                    lock (sockets.SyncRoot)
                    {
                        // \PbgXg炻ꂼ̃\Pbgo܂
                        foreach (Socket sck in sockets)
                        {
                            // \PbgI[vĂꍇ
                            if (sck.Connected)
                            {
                                // thɑ҂󂯒̃AhXǉ
                                ListenerName.AppendFormat(", {0}", sck.LocalEndPoint.ToString());
                            }
                        }
                    }
                }

                // thԂ
                return ListenerName.ToString();
            }
        }

        /// <summary>
        /// pPbg̑҂󂯂Ă邩ǂ\܂B
        /// </summary>
        public bool IsListening
        {
            get
            {
                return this.IsListen;
            }
        }
        #endregion

        #region IDisposable o
        /// <summary>
        /// CX^X̕ێĂ郊\[X܂B
        /// </summary>
        public void Dispose()
        {
            // }l[W\[XƃA}l[W\[Xj
            this.Dispose(true);

            // fXgN^̌Ăяo
            GC.SuppressFinalize(this);
        }
        #endregion

        /// <summary>
        /// 񓯊MŃXe[^X̕ۑɗpXe[gIuWFNgB
        /// </summary>
        private class ListenerState
        {
            private Socket socket = null;
            private byte[] buffer = null;
            private UInt16 bufferSize = UInt16.MaxValue;

            /// <summary>
            /// ڑe\[XGh|CgB
            /// </summary>
            public EndPoint TempEndPoint = null;

            /// <summary>
            /// RXgN^Bw肳ꂽAhbVOXL[ɊÂăCX^X쐬܂B
            /// </summary>
            /// <param name="ipVersion">\Pbgő҂󂯂AhbVOXL[B</param>
            public ListenerState(AddressFamily ipVersion)
            {
                // AhbVOXL[ɂăobt@TCYK؂ɐݒ肷
                switch (ipVersion)
                {
                    // IPv4AhbVOXL[̏ꍇ
                    case AddressFamily.InterNetwork:
                    {
                        // \PbgIPv4T|[gĂȂꍇ
                        if (!Socket.SupportsIPv4)
                        {
                            // OX[
                            throw new ArgumentException("݂̊ IPv4 T|[gĂ܂B");
                        }

                        // ׂĂIPv4\[XAhXe
                        TempEndPoint = (EndPoint) new IPEndPoint(IPAddress.Any, 0);

                        // IPv4: 65535 - 28 oCgɐݒ肷
                        SetBufferSize(UInt16.MaxValue - 28);

                        break;
                    }
                    // IPv6AhbVOXL[̏ꍇ
                    case AddressFamily.InterNetworkV6:
                    {
                        // \PbgIPv6T|[gĂȂꍇ
                        if (!Socket.SupportsIPv6)
                        {
                            // OX[
                            throw new ArgumentException("݂̊ IPv6 T|[gĂ܂B");
                        }

                        // ׂĂIPv6\[XAhXe
                        TempEndPoint = (EndPoint) new IPEndPoint(IPAddress.IPv6Any, 0);

                        // IPv6: 65535 - 8 oCgɐݒ肷
                        SetBufferSize(UInt16.MaxValue - 8);

                        break;
                    }
                    // IPȊÕAhbVOXL[w肳ꂽꍇ
                    default:
                    {
                        // IPȊÕvgRw肳ꂽꍇ͗O𔭐
                        throw new ArgumentOutOfRangeException("ipVersion", ipVersion,
                            "ipVersion ̒l AddressFamily.InterNetwork ܂ AddressFamily.InterNetworkV6 łKv܂B");
                    }
                }
            }

            /// <summary>
            /// pPbg҂󂯃\Pbgݒ܂͎擾܂B
            /// </summary>
            public Socket Sockets
            {
                get
                {
                    return socket;
                }
                set
                {
                    if (value != null)
                        socket = value;
                    else
                        throw new ArgumentNullException("Sockets");
                }
            }

            /// <summary>
            /// pPbgǂݍރobt@ݒ܂͎擾܂B
            /// </summary>
            public byte[] Buffer
            {
                get
                {
                    return buffer;
                }
                set
                {
                    if (value != null)
                        buffer = value;
                    else
                        throw new ArgumentNullException("Buffer");
                }
            }

            /// <summary>
            /// ݒ肳Ăobt@TCY擾܂B
            /// </summary>
            public UInt16 BufferSize
            {
                get
                {
                    return bufferSize;
                }
            }

            /// <summary>
            /// obt@̃TCYݒ肵܂B
            /// </summary>
            /// <param name="size">ݒ肷obt@TCYB</param>
            private void SetBufferSize(UInt16 size)
            {
                bufferSize = size;

                // w肳ꂽobt@TCYŃobt@mۂ܂
                buffer = new Byte[BufferSize];
            }
        }
    }
}
