using System;
using System.Collections;
using System.Collections.Specialized;
using System.Diagnostics;
using Syslog.Config.SectionHandler;

namespace Syslog.Relay.Filter
{
	/// <summary>
	/// SyslogpPbgFacility, ServerityrSyslogpPbgtB^OtB^łB
	/// </summary>
	/// <remarks>g[XXCb`ɂătB^OpPbgg[Xo͂邩ł܂B
	/// <list type="table">
	/// <listheader>
	///     <term>g[X x</term>
	///     <description>\t@Cł̒l</description>
    /// </listheader>
    /// <item>
    ///     <term>Off</term>
    ///     <description>0</description>
    /// </item>
    /// <item>
    ///     <term>Info - tB^Og[Xo</term>
    ///     <description>3</description>
    /// </item>
    /// <item>
    ///     <term>Verbose - tB^OpPbg̓eg[Xo</term>
    ///     <description>4</description>
    /// </item>
	/// </list>
	/// </remarks>
	/// <example>
	/// <code>
	/// &lt;system.diagnostics&gt;
	///  &lt;switches&gt;
	///   &lt;add name="PriorityFilterTraceSwitch" value="3" /&gt;
    ///  &lt;/switches&gt;
	/// &lt;/system.diagnostics&gt;</code>
	/// </example>
	public class PriorityFilter : ISyslogRelayFilter
	{
        private const string ConfigTarget = "target";
        private const string ConfigFacility = "facility";
        private const string ConfigSeverity = "severity";

        private static TraceSwitch PriorityTracingSwitch = new TraceSwitch("PriorityFilterTraceSwitch", "Priority Filter ŃtB^OꂽpPbg̏@g[Xo͂邩肵܂");

        private ArrayList PriorityList = new ArrayList();

        /// <summary>
        /// ftHgRXgN^B
        /// </summary>
		public PriorityFilter()
		{
        }

        #region ISyslogRelayFilter o
        /// <summary>
        /// CxgfQ[gZbgAbvătB^܂B
        /// </summary>
        /// <param name="FilterEvent">CxgfQ[g܂ RelayFilterEventB</param>
        /// <param name="config">app.config JX^RtBO[Vm[hB</param>
        public void Init(RelayFilterEvent FilterEvent, string config)
        {
            this.LoadPrioritySetting(config);

            FilterEvent.Routing += new RoutingEventHandler(FilterEvent_Routing);
        }

        /// <summary>
        /// tB^̃thB
        /// </summary>
        public string Name
        {
            get
            {
                return "Priority Relay Filter";
            }
        }
        #endregion

        #region IDisposable o
        /// <summary>
        /// \[XJ܂B
        /// </summary>
        public void Dispose()
        {
        }
        #endregion

        /// <summary>
        /// tB^CxgMSyslogpPbg̃tB^Os܂B
        /// </summary>
        /// <param name="sender">Cxg̑MB</param>
        /// <param name="e">tB^OɕKvȃf[^i[ RoutingEventArgsB</param>
        /// <returns></returns>
        private SyslogInfo FilterEvent_Routing(object sender, RoutingEventArgs e)
        {
            // Facility擾܂
            int RoutingFacility = e.Syslog.Message.Facility;
            // Severity擾܂
            int RoutingSeverity = e.Syslog.Message.Severity;

            // tB^OOXg܂
            foreach (IDictionary col in PriorityList)
            {
                // tB^Õ[eBOvOCɌ肷ݒ擾
                string TargetValue = (string) col[ConfigTarget];

                // [eBO̃^[QbgvOCw肳Ăꍇ
                if (TargetValue != null)
                {
                    // w肵vOCł͂Ȃꍇ
                    if (TargetValue != e.PluginID)
                    {
                        // ̏OXgR[h̏ƍɐi
                        continue;
                    }
                }

                // Facility  Severity Ƃw肳Ăꍇ
                if (col[ConfigFacility] != null && col[ConfigSeverity] != null)
                {
                    // FacilityɈvSyslogbZ[Ŵ݂tB^ݒ擾
                    Facility FacilityValue = (Facility) col[ConfigFacility];
                    // Severityȏ̏dxSyslogbZ[Ŵ݂tB^ݒ擾
                    Severity SeverityValue = (Severity) col[ConfigSeverity];

                    // Facility ƍ
                    if (RoutingFacility == (int) FacilityValue)
                    {
                        // Severity wȏ̏dxƍ
                        if (RoutingSeverity <= (int) SeverityValue)
                        {
                            // O𖞂ꍇ͌㑱̃vOCփpPbg̏p
                            return e.Syslog;
                        }
                    }
                }
                // Facility ̂ݎw肳Ăꍇ
                else if (col[ConfigFacility] != null)
                {
                    // FacilityɈvSyslogbZ[Ŵ݂tB^ݒ擾
                    Facility FacilityValue = (Facility) col[ConfigFacility];

                    // Facility v邩ƍ
                    if (RoutingFacility == (int) FacilityValue)
                    {
                        // Facility vꍇ͌㑱̃vOCփpPbg̏p
                        return e.Syslog;
                    }
                }
                // Severity ̂ݎw肳Ăꍇ
                else if (col[ConfigSeverity] != null)
                {
                    // Severityȏ̏dxSyslogbZ[Ŵ݂tB^ݒ擾
                    Severity SeverityValue = (Severity) col[ConfigSeverity];

                    // Severity wȏ̏dxƍ
                    if (RoutingSeverity <= (int) SeverityValue)
                    {
                        // Severity wȏ̏dx̏ꍇ͌㑱֏p
                        return e.Syslog;
                    }
                }
            }
            
            // g[Xx Info ȏ̏ꍇ
            if (PriorityTracingSwitch.TraceInfo)
            {
                Trace.Indent();
                Trace.WriteLine(String.Format("[{0}], [{1}]: Message Filtered on PriorityFilter.", Priority.ToString((Facility) RoutingFacility), Priority.ToString((Severity) RoutingSeverity)), "Syslog.Relay.PriorityFilter");

                if (PriorityTracingSwitch.TraceVerbose)
                {
                    Trace.Indent();
                    Trace.WriteLine(e.Syslog.Message.ToString(), "Syslog.Relay.PriorityFilter");
                    Trace.Unindent();
                }

                Trace.Unindent();
            }

            // ׂĂ̏OXgɃ}b`Ȃꍇ͌㑱̏𒆎~
            return null;
        }

        /// <summary>
        /// JX^ݒ蕶ǂݍރtB^OOXg쐬܂B
        /// </summary>
        /// <param name="setting">app.config JX^RtBO[VB</param>
        private void LoadPrioritySetting(string setting)
        {
            // JX^ݒǂݍ
            ArrayList PrioritySetting = (ArrayList) CustomValueListSectionHandler.Parse(setting);

            // tB^OOXg܂
            foreach (Hashtable col in PrioritySetting)
            {
                // tB^Õ[eBOvOCɌ肷ݒ擾
                string TargetValue = (string) col[ConfigTarget];
                // FacilityɈvSyslogbZ[Ŵ݂tB^ݒ擾
                string FacilityValue = (string) col[ConfigFacility];
                // Severityȏ̏dxSyslogbZ[Ŵ݂tB^ݒ擾
                string SeverityValue = (string) col[ConfigSeverity];
                
                // OXgڃf[^i[ ListDictionary
                ListDictionary list = new ListDictionary();

                // KpvOCݒǉ
                list.Add(ConfigTarget, TargetValue);

                // Facility w肳Ăꍇ
                if (FacilityValue != null)
                {
                    //  Facility ɕϊ
                    Facility facility = Priority.ParseFacility(FacilityValue);

                    // Facility ݒǉ
                    list.Add(ConfigFacility, facility);
                }

                // Severity w肳Ăꍇ
                if (SeverityValue != null)
                {
                    //  Severity ɕϊ
                    Severity severity = Priority.ParseSeverity(SeverityValue);

                    // Severity ݒǉ
                    list.Add(ConfigSeverity, severity);
                }

                // OXgڃf[^OXgɒǉ
                PriorityList.Add(list);
            }
        }
    }
}
